@@ -24,6 +24,7 @@ struct IConnectionStream
     int direction;
     size_t elemSize;
     size_t elemMTU;
+    bool enabled;
 
     //rx cmd requests
     bool hasCmd;
@@ -138,6 +139,7 @@ SoapySDR::Stream *SoapyLMS7::setupStream(
             throw std::runtime_error("SoapyLMS7::setupStream() failed: " + std::string(GetLastErrorMessage()));
         stream->streamID.push_back(streamID);
         stream->elemMTU = _conn->GetStreamSize(streamID);
+        stream->enabled = false;
     }
 
     //calibrate these channels when activated
@@ -155,6 +157,13 @@ void SoapyLMS7::closeStream(SoapySDR::Stream *stream)
     auto icstream = (IConnectionStream *)stream;
     const auto &streamID = icstream->streamID;
 
+    //disable stream if left enabled
+    if (icstream->enabled)
+    {
+        for(auto i : streamID)
+            _conn->ControlStream(i, false);
+    }
+
     for(auto i : streamID)
         _conn->CloseStream(i);
 }
@@ -197,12 +206,16 @@ int SoapyLMS7::activateStream(
     icstream->numElems = numElems;
     icstream->hasCmd = true;
 
-    for(auto i : streamID)
+    if (not icstream->enabled)
     {
-        int status = _conn->ControlStream(i, true);
-        if(status != 0)
-            return SOAPY_SDR_STREAM_ERROR;
+        for(auto i : streamID)
+        {
+            int status = _conn->ControlStream(i, true);
+            if(status != 0) return SOAPY_SDR_STREAM_ERROR;
+        }
+        icstream->enabled = true;
     }
+
     return 0;
 }
 
@@ -220,12 +233,17 @@ int SoapyLMS7::deactivateStream(
     metadata.timestamp = SoapySDR::timeNsToTicks(timeNs, _conn->GetHardwareTimestampRate());
     metadata.hasTimestamp = (flags & SOAPY_SDR_HAS_TIME) != 0;
     metadata.endOfBurst = (flags & SOAPY_SDR_END_BURST) != 0;
-    for(auto i : streamID)
+
+    if (icstream->enabled)
     {
-        int status = _conn->ControlStream(i, false);
-        if(status != 0)
-            return SOAPY_SDR_STREAM_ERROR;
+        for(auto i : streamID)
+        {
+            int status = _conn->ControlStream(i, false);
+            if(status != 0) return SOAPY_SDR_STREAM_ERROR;
+        }
+        icstream->enabled = false;
     }
+
     return 0;
 }
 
@@ -384,10 +402,14 @@ int SoapyLMS7::readStreamStatus(
                 if (GetLastError() == EPERM) return SOAPY_SDR_NOT_SUPPORTED;
                 return SOAPY_SDR_TIMEOUT;
             }
+
+            //packet dropped doesnt mean anything for tx streams
+            if (icstream->direction == SOAPY_SDR_TX) metadata.packetDropped = false;
+
+            //stop when event is detected
+            if (metadata.endOfBurst || metadata.lateTimestamp || metadata.packetDropped)
+                goto found;
         }
-        //stop when event is detected
-        if (metadata.endOfBurst || metadata.lateTimestamp || metadata.packetDropped)
-            break;
         //check timeout
         std::chrono::duration<double> seconds = std::chrono::high_resolution_clock::now()-start;
         if (seconds.count()> (double)timeoutUs/1e6)
@@ -399,6 +421,7 @@ int SoapyLMS7::readStreamStatus(
             std::this_thread::sleep_for(std::chrono::microseconds(1+timeoutUs/2));
     }
 
+    found:
     timeNs = SoapySDR::ticksToTimeNs(metadata.timestamp, _conn->GetHardwareTimestampRate());
     //output metadata
     if (metadata.endOfBurst) flags |= SOAPY_SDR_END_BURST;
