@@ -9,6 +9,8 @@
 #include "VersionInfo.h"
 #include <assert.h>
 #include "FPGA_common.h"
+#include "Logger.h"
+#include "LMS64CProtocol.h"
 
 using namespace std;
 
@@ -733,7 +735,7 @@ API_EXPORT int CALL_CONV LMS_GetNormalizedGain(lms_device_t *device, bool dir_tx
     }
 
     *gain = lms->GetNormalizedGain(dir_tx,chan);
-    if (gain < 0)
+    if (*gain < 0)
         return -1;
     return LMS_SUCCESS;
 }
@@ -754,9 +756,10 @@ API_EXPORT int CALL_CONV LMS_GetGaindB(lms_device_t *device, bool dir_tx, size_t
         return -1;
     }
 
-    *gain = lms->GetGain(dir_tx,chan);
-    if (gain < 0)
+    int ret = lms->GetGain(dir_tx,chan);
+    if (ret < 0)
         return -1;
+    *gain = ret;
     return LMS_SUCCESS;
 }
 
@@ -1241,12 +1244,14 @@ API_EXPORT int CALL_CONV LMS_EnableTxWFM(lms_device_t *device, unsigned ch, bool
     uint16_t regAddr = 0x000D;
     uint16_t regValue = 0;
     int status = 0;
+    status = LMS_WriteFPGAReg(device, 0xFFFF, 1<<(ch/2));
+    if (status != 0)
+        return status;
     status = LMS_ReadFPGAReg(device, regAddr, &regValue);
     if(status != 0)
         return status;
-    unsigned offset = (ch/2)*2;
-    regValue = regValue & ~(0x6<<offset); //clear WFM_LOAD, WFM_PLAY
-    regValue |= (active << (1+offset));
+    regValue = regValue & ~0x6; //clear WFM_LOAD, WFM_PLAY
+    regValue |= active << 1;
     status = LMS_WriteFPGAReg(device, regAddr, regValue);
     return status;
 }
@@ -1259,15 +1264,15 @@ API_EXPORT int CALL_CONV LMS_GetStreamStatus(lms_stream_t *stream, lms_stream_st
         return -1;
     lime::IStreamChannel::Info info = channel->GetInfo();
 
-    status->active = false;
-    status->droppedPackets = 0;
+    status->active = info.active;
+    status->droppedPackets = info.droppedPackets;
     status->fifoFilledCount = info.fifoItemsCount;
     status->fifoSize = info.fifoSize;
     status->linkRate = info.linkRate;
-    status->overrun = 0;
-    status->underrun = 0;
-    status->sampleRate = 0;
-    status->timestamp = 0;
+    status->overrun = info.overrun;
+    status->underrun = info.underrun;
+    status->sampleRate = info.sampleRate;
+    status->timestamp = info.timestamp;
     return 0;
 }
 
@@ -1308,8 +1313,6 @@ API_EXPORT int CALL_CONV LMS_Program(lms_device_t *device, const char *data, siz
     return lms->Program(data,size,component,mode, callback);
 }
 
-
-
 API_EXPORT int CALL_CONV LMS_ProgramUpdate(lms_device_t *device, const bool download, lms_prog_callback_t callback)
 {
     if (device == nullptr)
@@ -1333,3 +1336,49 @@ API_EXPORT const char* LMS_GetLibraryVersion()
     sprintf(libraryVersion, "%.32s", lime::GetLibraryVersion().c_str());
     return libraryVersion;
 }
+
+static LMS_LogHandler api_msg_handler;
+static void APIMsgHandler(const lime::LogLevel level, const char *message)
+{
+    api_msg_handler(level,message);
+}
+
+API_EXPORT void LMS_RegisterLogHandler(LMS_LogHandler handler)
+{
+    lime::registerLogHandler(APIMsgHandler);
+    api_msg_handler = handler;
+}
+
+API_EXPORT int CALL_CONV LMS_TransferLMS64C(lms_device_t *dev, int cmd, uint8_t* data, size_t *len)
+{
+    if (dev == nullptr)
+    {
+        lime::ReportError(EINVAL, "Device cannot be NULL.");
+            return -1;
+    }
+    LMS7_Device* lms = (LMS7_Device*)dev;
+    lime::LMS64CProtocol::GenericPacket pkt;
+    auto conn = lms->GetConnection();
+    if (conn == nullptr)
+    {
+        lime::ReportError(EINVAL, "Device not connected");
+        return -1;
+    }
+
+    pkt.cmd = lime::eCMD_LMS(cmd);
+    for (size_t i = 0; i < *len; ++i)
+        pkt.outBuffer.push_back(data[i]);
+
+    lime::LMS64CProtocol* port = dynamic_cast<lime::LMS64CProtocol *>(conn);
+    if (port->TransferPacket(pkt) != 0)
+        return -1;
+
+    for (size_t i = 0; i < pkt.inBuffer.size(); ++i)
+        data[i] = pkt.inBuffer[i];
+    *len = pkt.inBuffer.size();
+
+    if (pkt.status != lime::STATUS_COMPLETED_CMD)
+        return lime::ReportError(-1, "%s", lime::status2string(pkt.status));
+
+    return 0;
+}
\ No newline at end of file
