@@ -20,15 +20,11 @@
 #include "LMS64CProtocol.h"
 #include <assert.h>
 #include "ConnectionRegistry.h"
+#include "ADF4002.h"
+#include "mcu_programs.h"
+#include "Logger.h"
 
-static const size_t LMS_PATH_NONE = 0;
-static const size_t LMS_PATH_HIGH = 1;
-static const size_t LMS_PATH_LOW = 2;
-static const size_t LMS_PATH_WIDE = 3;
-static const size_t LMS_PATH_TX1 = 1;
-static const size_t LMS_PATH_TX2 = 2;
-
-const double LMS7_Device::LMS_CGEN_MAX = 640000000;
+const double LMS7_Device::LMS_CGEN_MAX = 640e6;
 
 LMS7_Device* LMS7_Device::CreateDevice(lime::IConnection* conn, LMS7_Device *obj)
 {
@@ -339,7 +335,7 @@ size_t LMS7_Device::GetNumChannels(const bool tx) const
     return 2;
 }
 
-int LMS7_Device::SetRate(float_type f_Hz, int oversample)
+int LMS7_Device::SetRate(double f_Hz, int oversample)
 {
    int decim = 0;
    float_type nco_f=0;
@@ -364,6 +360,9 @@ int LMS7_Device::SetRate(float_type f_Hz, int oversample)
        }
    }
 
+    if (oversample == 0)
+        oversample =  LMS_CGEN_MAX/(8*f_Hz);
+
    if (oversample > 1)
    {
        for (decim = 0; decim < 4; decim++)
@@ -372,8 +371,6 @@ int LMS7_Device::SetRate(float_type f_Hz, int oversample)
                 break;
        }
    }
-   else if (oversample == 0)
-       decim = 0;
    else decim = 7;
 
    int ratio = oversample <= 2 ? 2 : (2<<decim);
@@ -412,23 +409,23 @@ int LMS7_Device::SetRate(float_type f_Hz, int oversample)
         float_type freq[LMS_NCO_VAL_COUNT]={0};
         if (rx_channels[i].cF_offset_nco != 0)
         {
-           freq[0] = rx_channels[i].cF_offset_nco;
+           freq[0] = fabs(rx_channels[i].cF_offset_nco);
            SetNCOFreq(false,i,freq,0);
-           SetNCO(false,i,0,true);
+           SetNCO(false,i,0,rx_channels[i].cF_offset_nco> 0);
         }
 
         if (tx_channels[i].cF_offset_nco != 0)
         {
-           freq[0] = tx_channels[i].cF_offset_nco;
+           freq[0] = fabs(tx_channels[i].cF_offset_nco);
            SetNCOFreq(true,i,freq,0);
-           SetNCO(true,i,0,true);
+           SetNCO(true,i,0,tx_channels[i].cF_offset_nco> 0);
         }
     }
 
     return 0;
 }
 
-int LMS7_Device::SetRate(bool tx, float_type f_Hz, size_t oversample)
+int LMS7_Device::SetRate(bool tx, double f_Hz, unsigned oversample)
 {
     float_type tx_clock;
     float_type rx_clock;
@@ -472,6 +469,9 @@ int LMS7_Device::SetRate(bool tx, float_type f_Hz, size_t oversample)
        }
    }
 
+    if (oversample == 0)
+        oversample = tx ? LMS_CGEN_MAX/(2*f_Hz) : LMS_CGEN_MAX/(8*f_Hz);
+
     if (oversample > 1)
     {
         for (tmp = 0; tmp < 4; tmp++)
@@ -482,8 +482,6 @@ int LMS7_Device::SetRate(bool tx, float_type f_Hz, size_t oversample)
             }
         }
     }
-    else if (oversample == 0)
-        tmp = 0;
     else tmp = 7;
 
     int ratio = oversample <= 2 ? 2 : (2<<tmp);
@@ -690,23 +688,29 @@ int LMS7_Device::SetRate(bool tx, float_type f_Hz, size_t oversample)
         float_type freq[LMS_NCO_VAL_COUNT]={0};
         if (rx_channels[i].cF_offset_nco != 0)
         {
-           freq[0] = rx_channels[i].cF_offset_nco;
+           freq[0] = fabs(rx_channels[i].cF_offset_nco);
            SetNCOFreq(false,i,freq,0);
-           SetNCO(false,i,0,true);
+           SetNCO(false,i,0,rx_channels[i].cF_offset_nco> 0);
         }
 
         if (tx_channels[i].cF_offset_nco != 0)
         {
-           freq[0] = tx_channels[i].cF_offset_nco;
+           freq[0] = fabs(tx_channels[i].cF_offset_nco);
            SetNCOFreq(true,i,freq,0);
-           SetNCO(true,i,0,true);
+           SetNCO(true,i,0,tx_channels[i].cF_offset_nco> 0);
         }
     }
 
    return 0;
 }
 
-double LMS7_Device::GetRate(bool tx, size_t chan,float_type *rf_rate_Hz)
+int LMS7_Device::SetRate(unsigned ch, double rxRate, double txRate, unsigned oversample)
+{
+    //TODO: low importance : implement this and expose via LimeSuite.h
+    return lime::ReportError(ERANGE, "Not implemented");;
+}
+
+double LMS7_Device::GetRate(bool tx, unsigned chan, double *rf_rate_Hz)
 {
 
     double interface_Hz;
@@ -717,12 +721,12 @@ double LMS7_Device::GetRate(bool tx, size_t chan,float_type *rf_rate_Hz)
 
     if (tx)
     {
-        ratio = lms->Get_SPI_Reg_bits(LMS7param(HBI_OVR_TXTSP));
+        ratio = lms->Get_SPI_Reg_bits(LMS7param(HBI_OVR_TXTSP),true);
         interface_Hz = lms->GetReferenceClk_TSP(lime::LMS7002M::Tx);
     }
     else
     {
-        ratio = lms->Get_SPI_Reg_bits(LMS7param(HBD_OVR_RXTSP));
+        ratio = lms->Get_SPI_Reg_bits(LMS7param(HBD_OVR_RXTSP),true);
         interface_Hz = lms->GetReferenceClk_TSP(lime::LMS7002M::Rx);
     }
 
@@ -741,8 +745,8 @@ double LMS7_Device::GetRate(bool tx, size_t chan,float_type *rf_rate_Hz)
 lms_range_t LMS7_Device::GetRxRateRange(const size_t chan) const
 {
   lms_range_t ret;
-  ret.max = 30000000;
-  ret.min = 100000;
+  ret.max = 80e6;
+  ret.min = 100e3;
   ret.step = 1;
   return ret;
 }
@@ -750,8 +754,8 @@ lms_range_t LMS7_Device::GetRxRateRange(const size_t chan) const
 lms_range_t LMS7_Device::GetTxRateRange(size_t chan) const
 {
   lms_range_t ret;
-  ret.max = 30000000;
-  ret.min = 100000;
+  ret.max = 80e6;
+  ret.min = 100e3;
   ret.step = 1;
   return ret;
 }
@@ -809,7 +813,7 @@ lms_range_t LMS7_Device::GetRxPathBand(size_t path, size_t chan) const
   ret.step = 1;
   if (this->connection->GetDeviceInfo().deviceName == lime::GetDeviceName(lime::LMS_DEV_ULIMESDR))
   {
-      if (path == LMS_PATH_LOW)
+      if (path == LMS_PATH_LNAL)
       {
         ret.max = 3800000000;
         ret.min = 30000000;
@@ -824,16 +828,16 @@ lms_range_t LMS7_Device::GetRxPathBand(size_t path, size_t chan) const
   else
   switch (path)
   {
-      case LMS_PATH_HIGH:
+      case LMS_PATH_LNAH:
             ret.max = 3800000000;
             ret.min = 2000000000;
             break;
-      case LMS_PATH_WIDE:
-            ret.max = 3800000000;
+      case LMS_PATH_LNAW:
+            ret.max = 2600000000;
             ret.min = 700000000;
             break;
-      case LMS_PATH_LOW:
-            ret.max = 1100000000;
+      case LMS_PATH_LNAL:
+            ret.max = 1000000000;
             ret.min = 30000000;
             break;
       default:
@@ -870,10 +874,10 @@ lms_range_t LMS7_Device::GetTxPathBand(size_t path, size_t chan) const
   {
       case LMS_PATH_TX2:
             ret.max = 3800000000;
-            ret.min = 1500000000;
+            ret.min = 2000000000;
             break;
       case LMS_PATH_TX1:
-            ret.max = 1500000000;
+            ret.max = 2000000000;
             ret.min = 30000000;
             break;
       default:
@@ -1135,11 +1139,11 @@ int LMS7_Device::SetGFIR(bool tx, size_t chan, lms_gfir_t filt, bool enabled)
 
 int LMS7_Device::SetNormalizedGain(bool dir_tx, size_t chan,double gain)
 {
-    const int gain_total = 27 + 12 + 31;
+    const int gain_total = 27 + 12 + 31; //LNA + TIA + PGA
     if (dir_tx)
-        return SetGain(dir_tx,chan,63*gain+0.49);
+        return SetGain(dir_tx, chan, maxTxGain*gain+0.49);
     else
-        return SetGain(dir_tx,chan,gain*gain_total+0.49);
+        return SetGain(dir_tx, chan, gain*gain_total+0.49);
 }
 
 int LMS7_Device::SetGain(bool dir_tx, size_t chan, unsigned gain)
@@ -1148,16 +1152,32 @@ int LMS7_Device::SetGain(bool dir_tx, size_t chan, unsigned gain)
     if (lms->Modify_SPI_Reg_bits(LMS7param(MAC), (chan%2) + 1, true) != 0)
         return -1;
 
-    if (dir_tx)
+    if (dir_tx) //TX valid gain range 0-52
     {
-        if (gain > 63)
-            gain = 63;
-        if (lms->Modify_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),gain,true)!=0)
+        if (lms->CalibrateTxGain(0,nullptr)!=0) //find optimal BB gain
+            return -1;
+        //From datasheet: TXPAD: 0<=Loss<=10 – Pout=Pout_max-Loss; 11<=Loss<31 – Pout=Pout_max-10-2*(Loss-10)
+        //therefore MAX TDPAD gain: 52 = 10+21*2
+        if (gain > 52) //increase BB gain over optimal, can cause saturation in amp
+        {
+            int gbb = lms->Get_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),true);
+            double dgain = gain-52;
+            gain = (double)gbb*pow(10.0,dgain/20.0)+0.5;
+            lms->Modify_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),gain > 63 ? 63 : gain, true);
+            gain = 0;
+        }
+        else if (gain >= 42)
+            gain = 52-gain;
+        else
+            gain = 31-gain/2;
+        if (lms->Modify_SPI_Reg_bits(LMS7param(LOSS_LIN_TXPAD_TRF),gain,true)!=0)
+            return -1;
+        if (lms->Modify_SPI_Reg_bits(LMS7param(LOSS_MAIN_TXPAD_TRF),gain,true)!=0)
             return -1;
     }
-    else
+    else //Rx valid gain range 0-70
     {
-        if (gain > 70)
+        if (gain > 70) //do not exceed gain table index
             gain = 70;
         unsigned lna = 0, tia = 0, pga = 0;
         //LNA table
@@ -1186,7 +1206,7 @@ int LMS7_Device::SetGain(bool dir_tx, size_t chan, unsigned gain)
         lna = lnaTbl[gain];
         pga = pgaTbl[gain];
 
-        int rcc_ctl_pga_rbb = (430*(pow(0.65,((double)pga/10)))-110.35)/20.4516+16;
+        int rcc_ctl_pga_rbb = (430*(pow(0.65,((double)pga/10)))-110.35)/20.4516+16; //from datasheet
 
         if ((lms->Modify_SPI_Reg_bits(LMS7param(G_LNA_RFE),lna+1,true)!=0)
           ||(lms->Modify_SPI_Reg_bits(LMS7param(G_TIA_RFE),tia+1,true)!=0)
@@ -1199,10 +1219,10 @@ int LMS7_Device::SetGain(bool dir_tx, size_t chan, unsigned gain)
 
 double LMS7_Device::GetNormalizedGain(bool dir_tx, size_t chan)
 {
-    const double gain_total = 27 + 12 + 31;
+    const double gain_total = 27 + 12 + 31; //LNA + TIA + PGA
     double ret = GetGain(dir_tx, chan);
     if (dir_tx)
-        return ret / 63.0;
+        return ret > maxTxGain ? 1.0 : ret / maxTxGain;
     else
         return ret / gain_total;
 }
@@ -1214,8 +1234,20 @@ int LMS7_Device::GetGain(bool dir_tx, size_t chan)
         return -1;
     if (dir_tx)
     {
-        int ret = lms->Get_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),true);
-        return ret;
+        int gain = lms->Get_SPI_Reg_bits(LMS7param(LOSS_MAIN_TXPAD_TRF),true);
+        if (gain <= 10)
+            gain = 52-gain;
+        else
+            gain = 62-gain*2;
+        int bak = lms->Get_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),true); //backup
+        if (lms->CalibrateTxGain(0,nullptr)!=0)
+            return -1;
+        int opt = lms->Get_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),true);
+        gain += 20.0*log10((double)bak / (double) opt)+0.5;
+
+        lms->Modify_SPI_Reg_bits(LMS7param(CG_IAMP_TBB),bak, true); //restore
+
+        return gain;
     }
     else
     {
@@ -1531,22 +1563,47 @@ int LMS7_Device::Calibrate(bool dir_tx, size_t chan, double bw, unsigned flags)
 int LMS7_Device::SetRxFrequency(size_t chan, double f_Hz)
 {
     lime::LMS7002M* lms = lms_list[chan / 2];
+    rx_channels[chan].freq = f_Hz;
+    int chA = chan&(~1);
+    int chB = chan|1;
+    
+    if (rx_channels[chA].freq > 0 && rx_channels[chB].freq > 0) 
+    {
+        double delta = fabs(rx_channels[chA].freq - rx_channels[chB].freq);
+        if (delta > 1)
+        {
+            double rate = GetRate(false,chan);
+            if ((delta <= rate*31) && (delta+rate <= 160e6))
+            {
+                double center = (rx_channels[chA].freq+rx_channels[chB].freq)/2;
+                if (center < 30e6)
+                    center = 30e6;
+                rx_channels[chA].cF_offset_nco = center-rx_channels[chA].freq;
+                rx_channels[chB].cF_offset_nco = center-rx_channels[chB].freq;
+                if (lms->SetFrequencySX(false, center) != 0)
+                    return -1;
+                if (SetRate(false,rate,2)!=0)
+                    return -1;
+                return 0;
+            }
+        }
+    }
+    
     if (f_Hz < 30e6)
     {
         if (lms->SetFrequencySX(false, 30e6) != 0)
             return -1;
         rx_channels[chan].cF_offset_nco = 30e6-f_Hz;
-        if (SetRate(false,GetRate(true,chan),2)!=0)
-            return -1;
-    }
-    else
-    {
-        if (rx_channels[chan].cF_offset_nco != 0)
-            SetNCO(false,chan,-1,true);
-        rx_channels[chan].cF_offset_nco = 0;
-        if (lms->SetFrequencySX(false, f_Hz) != 0)
+        if (SetRate(false,GetRate(false,chan),2)!=0)
             return -1;
+        return 0;
     }
+
+    if (rx_channels[chan].cF_offset_nco != 0)
+        SetNCO(false,chan,-1,true);
+    rx_channels[chan].cF_offset_nco = 0;
+    if (lms->SetFrequencySX(false, f_Hz) != 0)
+        return -1;
     return 0;
 }
 
@@ -1560,6 +1617,32 @@ float_type LMS7_Device::GetTRXFrequency(bool tx, size_t chan)
 int LMS7_Device::SetTxFrequency(size_t chan, double f_Hz)
 {
     lime::LMS7002M* lms = lms_list[chan / 2];
+    tx_channels[chan].freq = f_Hz;
+    int chA = chan&(~1);
+    int chB = chan|1;
+    
+    if (tx_channels[chA].freq > 0 && tx_channels[chB].freq > 0) 
+    {
+        double delta = fabs(tx_channels[chA].freq - tx_channels[chB].freq);
+        if (delta > 1)
+        {
+            double rate = GetRate(true,chan);
+            if ((delta <= rate*31) && (delta+rate <= 640e6))
+            {
+                double center = (tx_channels[chA].freq+tx_channels[chB].freq)/2;
+                if (center < 30e6)
+                    center = 30e6;
+                tx_channels[chA].cF_offset_nco = center-tx_channels[chA].freq;
+                tx_channels[chB].cF_offset_nco = center-tx_channels[chB].freq;
+                if (lms->SetFrequencySX(true, center) != 0)
+                    return -1;
+                if (SetRate(true,rate,2)!=0)
+                    return -1;
+                return 0;
+            }
+        }
+    }
+    
     if (f_Hz < 30e6)
     {
         if (lms->SetFrequencySX(true, 30e6) != 0)
@@ -1567,15 +1650,14 @@ int LMS7_Device::SetTxFrequency(size_t chan, double f_Hz)
         tx_channels[chan].cF_offset_nco = 30e6-f_Hz;
         if (SetRate(true,GetRate(true,chan),2)!=0)
             return -1;
+        return 0;
     }
-    else
-    {
-        if (tx_channels[chan].cF_offset_nco != 0)
-            SetNCO(true,chan,-1,false);
-        tx_channels[chan].cF_offset_nco = 0;
-        if (lms->SetFrequencySX(true, f_Hz) != 0)
-            return -1;
-    }
+
+    if (tx_channels[chan].cF_offset_nco != 0)
+        SetNCO(true,chan,-1,false);
+    tx_channels[chan].cF_offset_nco = 0;
+    if (lms->SetFrequencySX(true, f_Hz) != 0)
+        return -1;
     return 0;
 }
 
@@ -1590,24 +1672,43 @@ lms_range_t LMS7_Device::GetFrequencyRange(bool tx) const
 
 int LMS7_Device::Init()
 {
-    this->Reset();
+    struct regVal
+    {
+        uint16_t adr;
+        uint16_t val;
+     };
+
+    const std::vector<regVal> initVals = {
+        {0x0022, 0x07FF}, {0x0023, 0x5550}, {0x002B, 0x0038}, {0x002C, 0x0000},
+        {0x002D, 0x0641}, {0x0086, 0x4101}, {0x0087, 0x5555}, {0x0088, 0x0525},
+        {0x0089, 0x1078}, {0x008B, 0x1F8C}, {0x008C, 0x267B}, {0x00A6, 0x000F},
+        {0x00A9, 0x8000}, {0x00AC, 0x2000}, {0x0108, 0x3026}, {0x010C, 0x8865},
+        {0x010E, 0x0000}, {0x0110, 0x0BFF}, {0x0113, 0x03C2}, {0x011C, 0xA941},
+        {0x011D, 0x0000}, {0x011E, 0x0984}, {0x0120, 0xB9FF}, {0x0121, 0x3650},
+        {0x0122, 0x033F}, {0x0123, 0x267B}, {0x0200, 0x00E1}, {0x0208, 0x0170},
+        {0x020B, 0x4000}, {0x020C, 0x8000}, {0x0400, 0x8081}, {0x040B, 0x1020},
+        {0x040C, 0x00F8}
+    };
+
     for (unsigned i = 0; i < lms_list.size(); i++)
     {
         lime::LMS7002M* lms = lms_list[i];
+        if (lms->ResetChip() != 0)
+            return -1;
 
-        lms->SetActiveChannel(lime::LMS7002M::ChA);
-        lms->Modify_SPI_Reg_bits(LMS7param(EN_ADCCLKH_CLKGN), 0);
-        lms->Modify_SPI_Reg_bits(LMS7param(CLKH_OV_CLKL_CGEN), 2);
-        lms->Modify_SPI_Reg_bits(LMS7param(LML1_MODE), 0);
-        lms->Modify_SPI_Reg_bits(LMS7param(LML2_MODE), 0);
-        lms->Modify_SPI_Reg_bits(LMS7param(PD_RX_AFE2), 0);
+        lms->Modify_SPI_Reg_bits(LMS7param(MAC), 1);
+        for (auto i : initVals)
+            lms->SPI_write(i.adr, i.val);
+
+        lms->Modify_SPI_Reg_bits(LMS7param(MAC), 2);
+        for (auto i : initVals)
+            if (i.adr >= 0x100)
+                lms->SPI_write(i.adr, i.val);
+        lms->EnableChannel(false, false);
+        lms->EnableChannel(true, false);
+
+        lms->Modify_SPI_Reg_bits(LMS7param(MAC), 1);
 
-        lms->SetActiveChannel(lime::LMS7002M::ChAB);
-        lms->Modify_SPI_Reg_bits(LMS7param(GFIR1_BYP_RXTSP), 1);
-        lms->Modify_SPI_Reg_bits(LMS7param(GFIR2_BYP_RXTSP), 1);
-        lms->Modify_SPI_Reg_bits(LMS7param(GFIR3_BYP_RXTSP), 1);
-        lms->Modify_SPI_Reg_bits(LMS7param(AGC_BYP_RXTSP), 1);
-        lms->Modify_SPI_Reg_bits(LMS7param(CMIX_BYP_RXTSP), 1);
         if (lms->UploadAll()!=0)
             return -1;
     }
@@ -1734,6 +1835,9 @@ int LMS7_Device::GetClockFreq(size_t clk_id, float_type *freq)
     case LMS_CLOCK_TXTSP:
         *freq = lms_list.at(lms_chip_id)->GetReferenceClk_TSP(true);
         return 0;
+    case LMS_CLOCK_EXTREF:
+        lime::ReportError(ENOTSUP, "Reading external reference clock is not supported");
+        return -1;
     default:
         lime::ReportError(EINVAL, "Invalid clock ID.");
         return -1;
@@ -1798,6 +1902,27 @@ int LMS7_Device::SetClockFreq(size_t clk_id, float_type freq)
     case LMS_CLOCK_TXTSP:
         lime::ReportError(ENOTSUP, "Setting TSP clocks is not supported.");
         return -1;
+    case LMS_CLOCK_EXTREF:
+        {
+            if (freq <= 0)
+            {
+                lime::ReportError(EINVAL, "Invalid frequency value.");
+                return -1;
+            }
+
+            lime::ADF4002 module;
+            module.SetDefaults();
+            double fvco = lms_list.at(lms_chip_id)->GetReferenceClk_SX(lime::LMS7002M::Rx);
+            int dummy;
+            module.SetFrefFvco(freq/1e6, fvco/1e6, dummy, dummy);
+            unsigned char data[12];
+            module.GetConfig(data);
+
+            std::vector<uint32_t> dataWr;
+            for(int i=0; i<12; i+=3)
+                dataWr.push_back((uint32_t)data[i] << 16 | (uint32_t)data[i+1] << 8 | data[i+2]);
+            return connection->TransactSPI(connection->GetDeviceInfo().addrADF4002, dataWr.data(), nullptr, 4);
+        }
     default:
         lime::ReportError(EINVAL, "Invalid clock ID.");
         return -1;
@@ -1948,6 +2073,34 @@ lime::LMS7002M* LMS7_Device::GetLMS(int index)
 
 int LMS7_Device::UploadWFM(const void **samples, uint8_t chCount, int sample_count, lime::StreamConfig::StreamDataFormat fmt)
 {
-    return connection->UploadWFM(samples, chCount%2 ? 1 : 2, sample_count, fmt, chCount>2 ? 1 : 0);
+    return connection->UploadWFM(samples, chCount%2 ? 1 : 2, sample_count, fmt, (chCount-1)/2);
 }
 
+int LMS7_Device::MCU_AGCStart(uint8_t rssiMin, uint8_t pgaCeil)
+{
+    lime::MCU_BD *mcu = lms_list.at(lms_chip_id)->GetMCUControls();
+    lms_list.at(lms_chip_id)->Modify_SPI_Reg_bits(0x0006, 0, 0, 0);
+
+    uint8_t mcuID = mcu->ReadMCUProgramID();
+    lime::debug("Current MCU firmware: %i, expected %i", mcuID, MCU_ID_AGC_IMAGE);
+    if(mcuID != MCU_ID_AGC_IMAGE)
+    {
+        lime::debug("Uploading MCU AGC firmware");
+        int status = mcu->Program_MCU(mcu_program_lms7_agc_bin, lime::IConnection::MCU_PROG_MODE::SRAM);
+        lime::info("MCU AGC firmware uploaded");
+        if(status != 0)
+            return status;
+    }
+
+    lms_list.at(lms_chip_id)->Modify_SPI_Reg_bits(0x002D, 15, 0, pgaCeil << 8 | rssiMin);
+    mcu->RunProcedure(254);
+    return 0;
+}
+
+int LMS7_Device::MCU_AGCStop()
+{
+    lime::MCU_BD *mcu = lms_list.at(lms_chip_id)->GetMCUControls();
+    mcu->RunProcedure(0);
+    lms_list.at(lms_chip_id)->Modify_SPI_Reg_bits(0x0006, 0, 0, 0);
+    return 0;
+}
