@@ -13,22 +13,18 @@
 #include <vector>
 #include <map>
 #include <string>
-#include <IConnection.h>
+#include "IConnection.h"
 
 class LIME_API LMS7_Device
 {
     class lms_channel_info
     {
     public:
-        lms_channel_info()
-        {
-            lpf_bw = 5e6;
-            cF_offset_nco = 0;
-            sample_rate = 30e6;
-        }
-        float_type lpf_bw;
-        float_type cF_offset_nco;
-        float_type sample_rate;
+        lms_channel_info():lpf_bw(5e6),cF_offset_nco(0),sample_rate(30e6),freq(0){}
+        double lpf_bw;
+        double cF_offset_nco;
+        double sample_rate;
+        double freq;
     };
 public:
     LMS7_Device(LMS7_Device *obj = nullptr);
@@ -39,9 +35,10 @@ public:
     int EnableChannel(bool dir_tx, size_t chan, bool enabled);
     int Reset();
     virtual size_t GetNumChannels(const bool tx=false) const;
-    int SetRate(float_type f_MHz, int oversample);
-    int SetRate(bool tx, float_type f_MHz, size_t oversample = 0);
-    float_type GetRate(bool tx,size_t chan,float_type *rf_rate_Hz = NULL);
+    virtual int SetRate(double f_MHz, int oversample);
+    virtual int SetRate(bool tx, double f_MHz, unsigned oversample = 0);
+    virtual int SetRate(unsigned ch, double rxRate, double txRate, unsigned oversample = 0);
+    virtual double GetRate(bool tx, unsigned chan, double *rf_rate_Hz = NULL);
     lms_range_t GetRxRateRange(const size_t chan = 0)const;
     lms_range_t GetTxRateRange(const size_t chan = 0)const;
     std::vector<std::string> GetPathNames(bool dir_tx, size_t chan) const;
@@ -94,7 +91,11 @@ public:
     int UploadWFM(const void **samples, uint8_t chCount, int sample_count, lime::StreamConfig::StreamDataFormat fmt);
     static LMS7_Device* CreateDevice(lime::IConnection* conn, LMS7_Device *obj = nullptr);
     std::map<std::string, double> extra_parameters;
+
+    int MCU_AGCStart(uint8_t rssiMin, uint8_t pgaCeil);
+    int MCU_AGCStop();
 protected:
+    const double maxTxGain = 60.0;
     lms_dev_info_t devInfo;
     std::vector<lms_channel_info> tx_channels;
     std::vector<lms_channel_info> rx_channels;
