@@ -79,7 +79,7 @@ int ConnectionSTREAM::UpdateExternalDataRate(const size_t channel, const double
     auto info = GetDeviceInfo();
     bool phaseSearch = false;
     if (!(mStreamers.size() > channel && (mStreamers[channel]->rxRunning || mStreamers[channel]->txRunning)))
-        if (this->chipVersion == 0x3841 && stoi(info.gatewareRevision) >= 7 && stoi(info.gatewareVersion) >= 2) //0x3840 LMS7002Mr2, 0x3841 LMS7002Mr3
+        if (this->chipVersion == 0x3841) //0x3840 LMS7002Mr2, 0x3841 LMS7002Mr3
             if(rxInterfaceClk >= 5e6 || txInterfaceClk >= 5e6)
                 phaseSearch = true;
 
@@ -190,15 +190,16 @@ int ConnectionSTREAM::ResetStreamBuffers()
 int ConnectionSTREAM::ReadRawStreamData(char* buffer, unsigned length, int epIndex, int timeout_ms)
 {
     const unsigned char ep = 0x81;
-    fpga::StopStreaming(this, epIndex);
+    WriteRegister(0xFFFF, 1 << epIndex);
+    fpga::StopStreaming(this);
 
     ResetStreamBuffers();
     WriteRegister(0x0008, 0x0100 | 0x2);
     WriteRegister(0x0007, 1);
-    fpga::StartStreaming(this, epIndex);
+    fpga::StartStreaming(this);
 
     int totalBytesReceived = ReceiveData(buffer,length, epIndex, timeout_ms);
-    fpga::StopStreaming(this, epIndex);
+    fpga::StopStreaming(this);
     AbortReading(ep);
 
     return totalBytesReceived;
@@ -211,18 +212,11 @@ void ConnectionSTREAM::ReceivePacketsLoop(Streamer* stream)
 {
     //at this point FPGA has to be already configured to output samples
     const uint8_t chCount = stream->mRxStreams.size();
-    const auto link = stream->mRxStreams[0]->config.linkFormat;
-    const uint32_t samplesInPacket = (link == StreamConfig::STREAM_12_BIT_COMPRESSED ? 1360 : 1020)/chCount;
+    const bool packed = stream->mRxStreams[0]->config.linkFormat == StreamConfig::STREAM_12_BIT_COMPRESSED;
+    const uint32_t samplesInPacket = (packed  ? 1360 : 1020)/chCount;
     const unsigned char ep = 0x81;
-    const int chipID = stream->mChipID;
 
-    float latency=0;
-    for (int i = 0; i < chCount; i++)
-        latency += stream->mRxStreams[i]->config.performanceLatency/chCount;
-
-    const unsigned tmp_cnt = (latency * 6)+0.5;
-
-    const uint8_t packetsToBatch = (1<<tmp_cnt);
+    const uint8_t packetsToBatch = stream->rxBatchSize;
     const uint32_t bufferSize = packetsToBatch*sizeof(FPGA_DataPacket);
     const uint8_t buffersCount = 16;
     vector<int> handles(buffersCount, 0);
@@ -238,12 +232,8 @@ void ConnectionSTREAM::ReceivePacketsLoop(Streamer* stream)
         return;
     }
 
-    int activeTransfers = 0;
     for (int i = 0; i<buffersCount; ++i)
-    {
         handles[i] = this->BeginDataReading(&buffers[i*bufferSize], bufferSize, ep);
-        ++activeTransfers;
-    }
 
     int bi = 0;
     unsigned long totalBytesReceived = 0; //for data rate calculation
@@ -275,36 +265,11 @@ void ConnectionSTREAM::ReceivePacketsLoop(Streamer* stream)
     uint64_t prevTs = 0;
     while (stream->terminateRx.load() == false)
     {
-        if(stream->generateData.load())
-        {
-            if(activeTransfers == 0) //stop FPGA when last transfer completes
-                fpga::StopStreaming(this, chipID);
-            stream->safeToConfigInterface.notify_all(); //notify that it's safe to change chip config
-            const int batchSize = (this->mExpectedSampleRate/chFrames[0].samplesCount)/10;
-            IStreamChannel::Metadata meta;
-            for(int i=0; i<batchSize; ++i)
-            {
-                for(int ch=0; ch<chCount; ++ch)
-                {
-                    meta.timestamp = chFrames[ch].timestamp;
-                    for(int j=0; j<chFrames[ch].samplesCount; ++j)
-                    {
-                        chFrames[ch].samples[j].i = 0;
-                        chFrames[ch].samples[j].q = 0;
-                    }
-                    uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, chFrames[ch].samplesCount, &meta);
-                    if(samplesPushed != chFrames[ch].samplesCount)
-                        lime::warning("Rx samples pushed %i/%i", samplesPushed, chFrames[ch].samplesCount);
-                }
-            }
-            this_thread::sleep_for(chrono::milliseconds(100));
-        }
         int32_t bytesReceived = 0;
         if(handles[bi] >= 0)
         {
             if (this->WaitForReading(handles[bi], 1000) == true)
                 bytesReceived = this->FinishDataReading(&buffers[bi*bufferSize], bufferSize, handles[bi]);
-            --activeTransfers;
             totalBytesReceived += bytesReceived;
             if (bytesReceived != int32_t(bufferSize)) //data should come in full sized packets
                 for(auto value: stream->mRxStreams)
@@ -322,7 +287,7 @@ void ConnectionSTREAM::ReceivePacketsLoop(Streamer* stream)
                     --resetFlagsDelay;
                 else
                 {
-                    lime::info("L %llu", (unsigned long long)pkt[pktIndex].counter);
+                    lime::info("L");
                     resetTxFlags.notify_one();
                     resetFlagsDelay = packetsToBatch*buffersCount;
                     stream->txLastLateTime.store(pkt[pktIndex].counter);
@@ -346,36 +311,22 @@ void ConnectionSTREAM::ReceivePacketsLoop(Streamer* stream)
             vector<complex16_t*> dest(chCount);
             for(uint8_t c=0; c<chCount; ++c)
                 dest[c] = (chFrames[c].samples);
-            size_t samplesCount = 0;
-            fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount, link, dest.data(), &samplesCount);
+            int samplesCount = fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount==2, packed, dest.data());
 
             for(int ch=0; ch<chCount; ++ch)
             {
                 IStreamChannel::Metadata meta;
                 meta.timestamp = pkt[pktIndex].counter;
-                meta.flags = RingFIFO::OVERWRITE_OLD;
-                uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
+                meta.flags = IStreamChannel::Metadata::OVERWRITE_OLD;
+                int samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
                 if(samplesPushed != samplesCount)
                     stream->mRxStreams[ch]->overflow++;
             }
         }
         // Re-submit this request to keep the queue full
-        if(not stream->generateData.load())
-        {
-            if(activeTransfers == 0) //reactivate FPGA and USB transfers
-                fpga::StartStreaming(this, chipID);
-            for(int i=0; i<buffersCount-activeTransfers; ++i)
-            {
-                handles[bi] = this->BeginDataReading(&buffers[bi*bufferSize], bufferSize, ep);
-                bi = (bi + 1) & (buffersCount-1);
-                ++activeTransfers;
-            }
-        }
-        else
-        {
-            handles[bi] = -1;
-            bi = (bi + 1) & (buffersCount-1);
-        }
+        handles[bi] = this->BeginDataReading(&buffers[bi*bufferSize], bufferSize, ep);
+        bi = (bi + 1) & (buffersCount-1);
+
         t2 = chrono::high_resolution_clock::now();
         auto timePeriod = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
         if (timePeriod >= 1000)
@@ -413,21 +364,15 @@ void ConnectionSTREAM::TransmitPacketsLoop(Streamer* stream)
     //at this point FPGA has to be already configured to output samples
     const uint8_t maxChannelCount = 2;
     const uint8_t chCount = stream->mTxStreams.size();
-    const auto link = stream->mTxStreams[0]->config.linkFormat;
+    const bool packed = stream->mTxStreams[0]->config.linkFormat == StreamConfig::STREAM_12_BIT_COMPRESSED;
     const unsigned char ep  = 0x01;
 
-    double latency=0;
-    for (int i = 0; i < chCount; i++)
-        latency += stream->mTxStreams[i]->config.performanceLatency/chCount;
-
-    const unsigned tmp_cnt = (latency * 6)+0.5;
-
     const uint8_t buffersCount = 16; // must be power of 2
-    const uint8_t packetsToBatch = (1<<tmp_cnt); //packets in single USB transfer
-    const uint32_t bufferSize = packetsToBatch*4096;
-    const uint32_t popTimeout_ms = 100;
+    const uint8_t packetsToBatch = stream->txBatchSize; //packets in single USB transfer
+    const uint32_t bufferSize = packetsToBatch*sizeof(FPGA_DataPacket);
+    const uint32_t popTimeout_ms = 500;
 
-    const int maxSamplesBatch = (link==StreamConfig::STREAM_12_BIT_COMPRESSED?1360:1020)/chCount;
+    const int maxSamplesBatch = (packed ? 1360:1020)/chCount;
     vector<int> handles(buffersCount, 0);
     vector<bool> bufferUsed(buffersCount, 0);
     vector<complex16_t> samples[maxChannelCount];
@@ -452,7 +397,7 @@ void ConnectionSTREAM::TransmitPacketsLoop(Streamer* stream)
     {
         if (bufferUsed[bi])
         {
-    	    unsigned bytesSent = 0; 
+    	    unsigned bytesSent = 0;
             if (this->WaitForSending(handles[bi], 1000) == true) {
                 bytesSent = this->FinishDataSending(&buffers[bi*bufferSize], bufferSize, handles[bi]);
 	    }
@@ -473,23 +418,25 @@ void ConnectionSTREAM::TransmitPacketsLoop(Streamer* stream)
         {
             IStreamChannel::Metadata meta;
             FPGA_DataPacket* pkt = reinterpret_cast<FPGA_DataPacket*>(&buffers[bi*bufferSize]);
-            bool badSamples = false;
             for(int ch=0; ch<chCount; ++ch)
             {
                 int samplesPopped = stream->mTxStreams[ch]->Read(samples[ch].data(), maxSamplesBatch, &meta, popTimeout_ms);
                 if (samplesPopped != maxSamplesBatch)
                 {
-                    badSamples = true;
+                    if (meta.flags & IStreamChannel::Metadata::END_BURST)
+                    {
+                        memset(&samples[ch][samplesPopped],0,maxSamplesBatch-samplesPopped);
+                        continue;
+                    }
                     stream->mTxStreams[ch]->underflow++;
-                    stream->txDataRate_Bps.store(0);
+                    stream->terminateTx.store(true);
 #ifndef NDEBUG
                     printf("popping from TX, samples popped %i/%i\n", samplesPopped, maxSamplesBatch);
 #endif
                     break;
                 }
             }
-            if (badSamples)
-                continue;
+
             if(stream->terminateTx.load() == true) //early termination
                 break;
             pkt[i].counter = meta.timestamp;
@@ -502,11 +449,13 @@ void ConnectionSTREAM::TransmitPacketsLoop(Streamer* stream)
             for(uint8_t c=0; c<chCount; ++c)
                 src[c] = (samples[c].data());
             uint8_t* const dataStart = (uint8_t*)pkt[i].data;
-            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount, link, dataStart, nullptr);
+            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount==2, packed, dataStart);
             ++i;
+            if (meta.flags & IStreamChannel::Metadata::END_BURST)
+                break;
         }
 
-        handles[bi] = this->BeginDataSending(&buffers[bi*bufferSize], bufferSize, ep);
+        handles[bi] = this->BeginDataSending(&buffers[bi*bufferSize], i*sizeof(FPGA_DataPacket), ep);
         bufferUsed[bi] = true;
 
         t2 = chrono::high_resolution_clock::now();
@@ -536,6 +485,7 @@ void ConnectionSTREAM::TransmitPacketsLoop(Streamer* stream)
         }
         bi = (bi + 1) & (buffersCount-1);
     }
+    stream->txRunning.store(false);
     stream->txDataRate_Bps.store(0);
 }
 
