@@ -15,6 +15,7 @@
 #include <ciso646>
 #include <FPGA_common.h>
 #include <ErrorReporting.h>
+#include "Logger.h"
 
 using namespace std;
 using namespace lime;
@@ -178,14 +179,14 @@ int ConnectionXillybus::UpdateExternalDataRate(const size_t channel, const doubl
 
 int ConnectionXillybus::ReadRawStreamData(char* buffer, unsigned length, int epIndex, int timeout_ms)
 {
-    fpga::StopStreaming(this, epIndex);
-
+    WriteRegister(0xFFFF, 1 << epIndex);
+    fpga::StopStreaming(this);
     ResetStreamBuffers();
     WriteRegister(0x0008, 0x0100 | 0x2);
     WriteRegister(0x0007, 1);
-    fpga::StartStreaming(this, epIndex);
+    fpga::StartStreaming(this);
     int totalBytesReceived = ReceiveData(buffer, length, epIndex, timeout_ms);
-    fpga::StopStreaming(this, epIndex);
+    fpga::StopStreaming(this);
     AbortReading(epIndex);
     return totalBytesReceived;
 }
@@ -199,18 +200,11 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
 {
     //at this point FPGA has to be already configured to output samples
     const uint8_t chCount = stream->mRxStreams.size();
-    const auto link = stream->mRxStreams[0]->config.linkFormat;
-    const uint32_t samplesInPacket = (link == StreamConfig::STREAM_12_BIT_COMPRESSED ? 1360 : 1020)/chCount;
+    const bool packed = stream->mRxStreams[0]->config.linkFormat== StreamConfig::STREAM_12_BIT_COMPRESSED;
+    const uint32_t samplesInPacket = (packed ? 1360 : 1020)/chCount;
     const int epIndex = stream->mChipID;
 
-    double latency=0;
-    for (int i = 0; i < chCount; i++)
-    {
-        latency += stream->mRxStreams[i]->config.performanceLatency/chCount;
-    }
-    const unsigned tmp_cnt = (latency * 6)+0.5;
-
-    const uint8_t packetsToBatch = (1<<tmp_cnt);
+    const uint8_t packetsToBatch = stream->rxBatchSize*2;
     const uint32_t bufferSize = packetsToBatch*sizeof(FPGA_DataPacket);
     vector<char>buffers(bufferSize, 0);
     vector<StreamChannel::Frame> chFrames;
@@ -225,15 +219,11 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
     }
 
     unsigned long totalBytesReceived = 0; //for data rate calculation
-    int m_bufferFailures = 0;
-    int32_t droppedSamples = 0;
-    int32_t packetLoss = 0;
-    bool generate_started = false;
 
     vector<uint32_t> samplesReceived(chCount, 0);
 
     auto t1 = chrono::high_resolution_clock::now();
-    auto t2 = chrono::high_resolution_clock::now();
+    auto t2 = t1;
 
     std::mutex txFlagsLock;
     condition_variable resetTxFlags;
@@ -259,37 +249,13 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
     uint64_t prevTs = 0;
     while (stream->terminateRx.load() == false)
     {
-        if(stream->generateData.load())
-        {
-            generate_started = true;
-            fpga::StopStreaming(this, epIndex);
-            stream->safeToConfigInterface.notify_all(); //notify that it's safe to change chip config
-            const int batchSize = (this->mExpectedSampleRate/chFrames[0].samplesCount)/10;
-            IStreamChannel::Metadata meta;
-            for(int i=0; i<batchSize; ++i)
-            {
-                for(int ch=0; ch<chCount; ++ch)
-                {
-                    meta.timestamp = chFrames[ch].timestamp;
-                    for(int j=0; j<chFrames[ch].samplesCount; ++j)
-                    {
-                        chFrames[ch].samples[j].i = 0;
-                        chFrames[ch].samples[j].q = 0;
-                    }
-                    uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, chFrames[ch].samplesCount, &meta);
-                    samplesReceived[ch] += chFrames[ch].samplesCount;
-                    if(samplesPushed != chFrames[ch].samplesCount)
-                        printf("Rx samples pushed %i/%i\n", samplesPushed, chFrames[ch].samplesCount);
-                }
-            }
-            this_thread::sleep_for(chrono::milliseconds(100));
-        }
         int32_t bytesReceived = 0;
 
         bytesReceived = this->ReceiveData(&buffers[0], bufferSize, epIndex, 1000);
         totalBytesReceived += bytesReceived;
         if (bytesReceived != int32_t(bufferSize)) //data should come in full sized packets
-            ++m_bufferFailures;
+            for(auto value: stream->mRxStreams)
+                value->underflow++;
 
         bool txLate=false;
         for (uint8_t pktIndex = 0; pktIndex < bytesReceived / sizeof(FPGA_DataPacket); ++pktIndex)
@@ -303,18 +269,23 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
                     --resetFlagsDelay;
                 else
                 {
-                    printf("L");
+                    lime::info("L");
                     resetTxFlags.notify_one();
                     resetFlagsDelay = packetsToBatch*2;
+                    stream->txLastLateTime.store(pkt[pktIndex].counter);
+                    for(auto value: stream->mTxStreams)
+                        value->pktLost++;
                 }
             }
             uint8_t* pktStart = (uint8_t*)pkt[pktIndex].data;
             if(pkt[pktIndex].counter - prevTs != samplesInPacket && pkt[pktIndex].counter != prevTs)
             {
+                int packetLoss = ((pkt[pktIndex].counter - prevTs)/samplesInPacket)-1;
 #ifndef NDEBUG
-                printf("\tRx pktLoss@%i - ts diff: %li  pktLoss: %.1f\n", pktIndex, pkt[pktIndex].counter - prevTs, float(pkt[pktIndex].counter - prevTs)/samplesInPacket);
+                printf("\tRx pktLoss: ts diff: %li  pktLoss: %i\n", pkt[pktIndex].counter - prevTs, packetLoss);
 #endif
-                packetLoss += (pkt[pktIndex].counter - prevTs)/samplesInPacket;
+                for(auto value: stream->mRxStreams)
+                    value->pktLost += packetLoss;
             }
             prevTs = pkt[pktIndex].counter;
             stream->rxLastTimestamp.store(pkt[pktIndex].counter);
@@ -322,24 +293,19 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
             vector<complex16_t*> dest(chCount);
             for(uint8_t c=0; c<chCount; ++c)
                 dest[c] = (chFrames[c].samples);
-            size_t samplesCount = 0;
-            fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount, link, dest.data(), &samplesCount);
+            int samplesCount = fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount==2, packed, dest.data());
 
             for(int ch=0; ch<chCount; ++ch)
             {
                 IStreamChannel::Metadata meta;
                 meta.timestamp = pkt[pktIndex].counter;
-                meta.flags = RingFIFO::OVERWRITE_OLD;
-                uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
+                meta.flags = IStreamChannel::Metadata::OVERWRITE_OLD;
+                int samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
                 if(samplesPushed != samplesCount)
-                    droppedSamples += samplesCount-samplesPushed;
+                    stream->mRxStreams[ch]->overflow++;;
             }
         }
-        // Re-submit this request to keep the queue full
-        if ((generate_started) && (!stream->generateData.load()))
-        {
-            fpga::StartStreaming(this, epIndex);
-        }
+
         t2 = chrono::high_resolution_clock::now();
         auto timePeriod = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
         if (timePeriod >= 1000)
@@ -348,16 +314,9 @@ void ConnectionXillybus::ReceivePacketsLoop(Streamer* stream)
             //total number of bytes sent per second
             double dataRate = 1000.0*totalBytesReceived / timePeriod;
 #ifndef NDEBUG
-            //each channel sample rate
-            float samplingRate = 1000.0*samplesReceived[0] / timePeriod;
-            printf("Rx: %.3f MB/s, Fs: %.3f MHz, overrun: %i, loss: %i \n", dataRate / 1000000.0, samplingRate / 1000000.0, droppedSamples, packetLoss);
+            printf("Rx: %.3f MB/s\n", dataRate / 1000000.0);
 #endif
-            samplesReceived[0] = 0;
             totalBytesReceived = 0;
-            m_bufferFailures = 0;
-            droppedSamples = 0;
-            packetLoss = 0;
-
             stream->rxDataRate_Bps.store((uint32_t)dataRate);
         }
     }
@@ -377,20 +336,13 @@ void ConnectionXillybus::TransmitPacketsLoop(Streamer* stream)
     //at this point FPGA has to be already configured to output samples
     const uint8_t maxChannelCount = 2;
     const uint8_t chCount = stream->mTxStreams.size();
-    const auto link = stream->mTxStreams[0]->config.linkFormat;
+    const bool packed = stream->mTxStreams[0]->config.linkFormat==StreamConfig::STREAM_12_BIT_COMPRESSED;
     const int epIndex = stream->mChipID;
 
-    double latency=0;
-    for (int i = 0; i < chCount; i++)
-    {
-        latency += stream->mTxStreams[i]->config.performanceLatency/chCount;
-    }
-    const unsigned tmp_cnt = (latency * 6)+0.5;
-    const uint8_t packetsToBatch = (1<<tmp_cnt); //packets in single USB transfer
+    const uint8_t packetsToBatch = stream->txBatchSize*2;; //packets in single USB transfer
     const uint32_t bufferSize = packetsToBatch*4096;
-    const uint32_t popTimeout_ms = 100;
-
-    const int maxSamplesBatch = (link==StreamConfig::STREAM_12_BIT_COMPRESSED?1360:1020)/chCount;
+    const uint32_t popTimeout_ms = 500;
+    const int maxSamplesBatch = (packed ? 1360:1020)/chCount;
     vector<complex16_t> samples[maxChannelCount];
     vector<char> buffers;
     try
@@ -401,17 +353,13 @@ void ConnectionXillybus::TransmitPacketsLoop(Streamer* stream)
     }
     catch (const std::bad_alloc& ex) //not enough memory for buffers
     {
-        printf("Error allocating Tx buffers, not enough memory\n");
+        lime::error("Error allocating Tx buffers, not enough memory");
         return;
     }
 
-    int m_bufferFailures = 0;
     long totalBytesSent = 0;
-
-    uint32_t samplesSent = 0;
-
     auto t1 = chrono::high_resolution_clock::now();
-    auto t2 = chrono::high_resolution_clock::now();
+    auto t2 = t1;
 
     while (stream->terminateTx.load() != true)
     {
@@ -426,9 +374,17 @@ void ConnectionXillybus::TransmitPacketsLoop(Streamer* stream)
                 int samplesPopped = stream->mTxStreams[ch]->Read(samples[ch].data(), maxSamplesBatch, &meta, popTimeout_ms);
                 if (samplesPopped != maxSamplesBatch)
                 {
-                #ifndef NDEBUG
-                    printf("Warning popping from TX, samples popped %i/%i\n", samplesPopped, maxSamplesBatch);
-                #endif
+                    if (meta.flags & IStreamChannel::Metadata::END_BURST)
+                    {
+                        memset(&samples[ch][samplesPopped],0,maxSamplesBatch-samplesPopped);
+                        continue;
+                    }
+                    stream->mTxStreams[ch]->underflow++;
+                    stream->terminateTx.store(true);
+#ifndef NDEBUG
+                    printf("popping from TX, samples popped %i/%i\n", samplesPopped, maxSamplesBatch);
+#endif
+                    break;
                 }
             }
             if(stream->terminateTx.load() == true) //early termination
@@ -443,15 +399,17 @@ void ConnectionXillybus::TransmitPacketsLoop(Streamer* stream)
             for(uint8_t c=0; c<chCount; ++c)
                 src[c] = (samples[c].data());
             uint8_t* const dataStart = (uint8_t*)pkt[i].data;
-            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount, link, dataStart, nullptr);
-            samplesSent += maxSamplesBatch;
+            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount==2, packed, dataStart);
             ++i;
         }
 
         uint32_t bytesSent = this->SendData(&buffers[0], bufferSize, epIndex, 1000);
-                totalBytesSent += bytesSent;
-        if (bytesSent != bufferSize)
-            ++m_bufferFailures;
+        if (bytesSent != bufferSize){
+            for (auto value : stream->mTxStreams)
+		value->overflow++;
+        }
+        else
+            totalBytesSent += bytesSent;
 
         t2 = chrono::high_resolution_clock::now();
         auto timePeriod = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
@@ -460,19 +418,16 @@ void ConnectionXillybus::TransmitPacketsLoop(Streamer* stream)
             //total number of bytes sent per second
             float dataRate = 1000.0*totalBytesSent / timePeriod;
             stream->txDataRate_Bps.store(dataRate);
-            m_bufferFailures = 0;
-            samplesSent = 0;
             totalBytesSent = 0;
             t1 = t2;
 #ifndef NDEBUG
-            //total number of samples from all channels per second
-            float sampleRate = 1000.0*samplesSent / timePeriod;
-            printf("Tx: %.3f MB/s, Fs: %.3f MHz, failures: %i\n", dataRate / 1000000.0, sampleRate / 1000000.0, m_bufferFailures);
+            printf("Tx: %.3f MB/s\n", dataRate / 1000000.0);
 #endif
         }
     }
 
     // Wait for all the queued requests to be cancelled
     AbortSending(epIndex);
+    stream->txRunning.store(false);
     stream->txDataRate_Bps.store(0);
 }
