@@ -16,6 +16,7 @@
 #include <vector>
 #include <FPGA_common.h>
 #include "ErrorReporting.h"
+#include "Logger.h"
 
 using namespace lime;
 using namespace std;
@@ -135,9 +136,6 @@ int Connection_uLimeSDR::UpdateExternalDataRate(const size_t channel, const doub
         if (phaseSearch)
         {
             {
-#ifndef NDEBUG
-                printf("RX phase config:\n");
-#endif
                 clocks[3].findPhase = true;
                 const std::vector<uint32_t> spiData = { 0x0E9F, 0x07FF, 0x5550, 0xE4E4,
                     0xE4E4, 0x0086, 0x028D, 0x00FF, 0x5555, 0x02CD, 0xAAAA, 0x02ED };
@@ -149,9 +147,6 @@ int Connection_uLimeSDR::UpdateExternalDataRate(const size_t channel, const doub
                 status = lime::fpga::SetPllFrequency(this, 0, rxInterfaceClk, clocks, 4);
             }
             {
-#ifndef NDEBUG
-                printf("TX phase config:\n");
-#endif
                 clocks[3].findPhase = false;
                 const std::vector<uint32_t> spiData = { 0x0E9F, 0x07FF, 0x5550, 0xE4E4, 0xE4E4, 0x0484 };
                 WriteRegister(0x000A, 0x0000);
@@ -191,20 +186,20 @@ int Connection_uLimeSDR::UpdateExternalDataRate(const size_t channel, const doub
 int Connection_uLimeSDR::ReadRawStreamData(char* buffer, unsigned length, int epIndex, int timeout_ms)
 {
     int totalBytesReceived = 0;
-    fpga::StopStreaming(this, epIndex);
+    fpga::StopStreaming(this);
 
     //ResetStreamBuffers();
     WriteRegister(0x0008, 0x0100 | 0x2);
     WriteRegister(0x0007, 1);
 
-    fpga::StartStreaming(this, epIndex);
+    fpga::StartStreaming(this);
 
     int handle = BeginDataReading(buffer, length);
     if (WaitForReading(handle, timeout_ms))
         totalBytesReceived = FinishDataReading(buffer, length, handle);
 
     AbortReading();
-    fpga::StopStreaming(this, epIndex);
+    fpga::StopStreaming(this);
 
     return totalBytesReceived;
 }
@@ -235,9 +230,8 @@ void Connection_uLimeSDR::ReceivePacketsLoop(Connection_uLimeSDR::Streamer* stre
 {
     //at this point FPGA has to be already configured to output samples
     const uint8_t chCount = stream->mRxStreams.size();
-    const auto link =stream->mRxStreams[0]->config.linkFormat;
-    const uint32_t samplesInPacket = (link == StreamConfig::STREAM_12_BIT_COMPRESSED ? 1360 : 1020)/chCount;
-    const int chipID = stream->mChipID;
+    const bool packed = stream->mRxStreams[0]->config.linkFormat == StreamConfig::STREAM_12_BIT_COMPRESSED;
+    const uint32_t samplesInPacket = (packed ? 1360 : 1020)/chCount;
 
     double latency=0;
     for (int i = 0; i < chCount; i++)
@@ -262,12 +256,8 @@ void Connection_uLimeSDR::ReceivePacketsLoop(Connection_uLimeSDR::Streamer* stre
         return;
     }
 
-    uint8_t activeTransfers = 0;
     for (int i = 0; i<buffersCount; ++i)
-    {
         handles[i] = this->BeginDataReading(&buffers[i*bufferSize], bufferSize);
-        ++activeTransfers;
-    }
 
     int bi = 0;
     unsigned long totalBytesReceived = 0; //for data rate calculation
@@ -305,38 +295,12 @@ void Connection_uLimeSDR::ReceivePacketsLoop(Connection_uLimeSDR::Streamer* stre
     uint64_t prevTs = 0;
     while (stream->terminateRx.load() == false)
     {
-        if(stream->generateData.load())
-        {
-            if(activeTransfers == 0) //stop FPGA when last transfer completes
-                fpga::StopStreaming(this, chipID);
-            stream->safeToConfigInterface.notify_all(); //notify that it's safe to change chip config
-            const int batchSize = (this->mExpectedSampleRate/chFrames[0].samplesCount)/10;
-            IStreamChannel::Metadata meta;
-            for(int i=0; i<batchSize; ++i)
-            {
-                for(int ch=0; ch<chCount; ++ch)
-                {
-                    meta.timestamp = chFrames[ch].timestamp;
-                    for(int j=0; j<chFrames[ch].samplesCount; ++j)
-                    {
-                        chFrames[ch].samples[j].i = 0;
-                        chFrames[ch].samples[j].q = 0;
-                    }
-                    uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, chFrames[ch].samplesCount, &meta);
-                    samplesReceived[ch] += chFrames[ch].samplesCount;
-                    if(samplesPushed != chFrames[ch].samplesCount)
-                        printf("Rx samples pushed %i/%i\n", samplesPushed, chFrames[ch].samplesCount);
-                }
-            }
-            this_thread::sleep_for(chrono::milliseconds(100));
-        }
         int32_t bytesReceived = 0;
         if(handles[bi] >= 0)
         {
             if (this->WaitForReading(handles[bi], 1000) == false)
                 ++m_bufferFailures;
             bytesReceived = this->FinishDataReading(&buffers[bi*bufferSize], bufferSize, handles[bi]);
-            --activeTransfers;
             totalBytesReceived += bytesReceived;
             if (bytesReceived != int32_t(bufferSize)) //data should come in full sized packets
                 ++m_bufferFailures;
@@ -353,7 +317,7 @@ void Connection_uLimeSDR::ReceivePacketsLoop(Connection_uLimeSDR::Streamer* stre
                     --resetFlagsDelay;
                 else
                 {
-                    printf("L");
+                    lime::warning("L");
                     resetTxFlags.notify_one();
                     resetFlagsDelay = packetsToBatch*buffersCount;
                     stream->txLastLateTime.store(pkt[pktIndex].counter);
@@ -373,36 +337,22 @@ void Connection_uLimeSDR::ReceivePacketsLoop(Connection_uLimeSDR::Streamer* stre
             vector<complex16_t*> dest(chCount);
             for(uint8_t c=0; c<chCount; ++c)
                 dest[c] = (chFrames[c].samples);
-            size_t samplesCount = 0;
-            fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount, link, dest.data(), &samplesCount);
+            int samplesCount = fpga::FPGAPacketPayload2Samples(pktStart, 4080, chCount==2, packed, dest.data());
 
             for(int ch=0; ch<chCount; ++ch)
             {
                 IStreamChannel::Metadata meta;
                 meta.timestamp = pkt[pktIndex].counter;
-                meta.flags = RingFIFO::OVERWRITE_OLD;
-                uint32_t samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
+                meta.flags = IStreamChannel::Metadata::OVERWRITE_OLD;
+                int samplesPushed = stream->mRxStreams[ch]->Write((const void*)chFrames[ch].samples, samplesCount, &meta, 100);
                 if(samplesPushed != samplesCount)
                     droppedSamples += samplesCount-samplesPushed;
             }
         }
         // Re-submit this request to keep the queue full
-        if(not stream->generateData.load())
-        {
-            if(activeTransfers == 0) //reactivate FPGA and USB transfers
-                fpga::StartStreaming(this, chipID);
-            for(int i=0; i<buffersCount-activeTransfers; ++i)
-            {
-                handles[bi] = this->BeginDataReading(&buffers[bi*bufferSize], bufferSize);
-                bi = (bi + 1) & (buffersCount-1);
-                ++activeTransfers;
-            }
-        }
-        else
-        {
-            handles[bi] = -1;
-            bi = (bi + 1) & (buffersCount-1);
-        }
+        handles[bi] = this->BeginDataReading(&buffers[bi*bufferSize], bufferSize);
+        bi = (bi + 1) & (buffersCount-1);
+
         t2 = chrono::high_resolution_clock::now();
         auto timePeriod = std::chrono::duration_cast<std::chrono::milliseconds>(t2 - t1).count();
         if (timePeriod >= 1000)
@@ -448,7 +398,7 @@ void Connection_uLimeSDR::TransmitPacketsLoop(Streamer* stream)
     //at this point FPGA has to be already configured to output samples
     const uint8_t maxChannelCount = 2;
     const uint8_t chCount = stream->mTxStreams.size();
-    const auto link = stream->mTxStreams[0]->config.linkFormat;
+    const bool packed = stream->mTxStreams[0]->config.linkFormat==StreamConfig::STREAM_12_BIT_COMPRESSED;
 
     double latency=0;
     for (int i = 0; i < chCount; i++)
@@ -463,7 +413,7 @@ void Connection_uLimeSDR::TransmitPacketsLoop(Streamer* stream)
     const uint32_t bufferSize = packetsToBatch*4096;
     const uint32_t popTimeout_ms = 100;
 
-    const int maxSamplesBatch = (link==StreamConfig::STREAM_12_BIT_COMPRESSED?1360:1020)/chCount;
+    const int maxSamplesBatch = (packed ? 1360:1020)/chCount;
     vector<int> handles(buffersCount, 0);
     vector<bool> bufferUsed(buffersCount, 0);
     vector<uint32_t> bytesToSend(buffersCount, 0);
@@ -477,7 +427,7 @@ void Connection_uLimeSDR::TransmitPacketsLoop(Streamer* stream)
     }
     catch (const std::bad_alloc& ex) //not enough memory for buffers
     {
-        printf("Error allocating Tx buffers, not enough memory\n");
+        lime::error("Error allocating Tx buffers, not enough memory");
         return;
     }
 
@@ -513,9 +463,17 @@ void Connection_uLimeSDR::TransmitPacketsLoop(Streamer* stream)
                 int samplesPopped = stream->mTxStreams[ch]->Read(samples[ch].data(), maxSamplesBatch, &meta, popTimeout_ms);
                 if (samplesPopped != maxSamplesBatch)
                 {
-                #ifndef NDEBUG
-                    printf("Warning popping from TX, samples popped %i/%i\n", samplesPopped, maxSamplesBatch);
-                #endif
+                    if (meta.flags & IStreamChannel::Metadata::END_BURST)
+                    {
+                        memset(&samples[ch][samplesPopped],0,maxSamplesBatch-samplesPopped);
+                        continue;
+                    }
+                    stream->mTxStreams[ch]->underflow++;
+                    stream->terminateTx.store(true);
+#ifndef NDEBUG
+                    printf("popping from TX, samples popped %i/%i\n", samplesPopped, maxSamplesBatch);
+#endif
+                    break;
                 }
 
             }
@@ -531,7 +489,7 @@ void Connection_uLimeSDR::TransmitPacketsLoop(Streamer* stream)
             for(uint8_t c=0; c<chCount; ++c)
                 src[c] = (samples[c].data());
             uint8_t* const dataStart = (uint8_t*)pkt[i].data;
-            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount, link, dataStart, nullptr);
+            fpga::Samples2FPGAPacketPayload(src.data(), maxSamplesBatch, chCount==2, packed, dataStart);
             samplesSent += maxSamplesBatch;
             ++i;
         }
