@@ -7,7 +7,7 @@
 #ifndef LIMESUITE_VERSION_INFO_H
 #define LIMESUITE_VERSION_INFO_H
 
-#include <LimeSuiteConfig.h>
+#include "LimeSuiteConfig.h"
 #include <string>
 
 /*!
@@ -24,7 +24,7 @@
  * #endif
  * \endcode
  */
-#define LIME_SUITE_API_VERSION 0x20170600
+#define LIME_SUITE_API_VERSION 0x20170900
 
 namespace lime
 {
