@@ -25,6 +25,7 @@
 #include <chrono>
 #include <thread>
 #include "Logger.h"
+#include "mcu_programs.h"
 
 #include "MCU_BD.h"
 const static uint16_t MCU_PARAMETER_ADDRESS = 0x002D; //register used to pass parameter values to MCU
@@ -126,7 +127,7 @@ It requires IConnection to be set by SetConnection() to communicate with chip
 */
 LMS7002M::LMS7002M() :
     useCache(0),
-    mValueCache(new CalibrationCache()),
+    mValueCache(nullptr),
     mRegistersMap(new LMS7002M_RegistersMap()),
     controlPort(nullptr),
     mdevIndex(0),
@@ -193,6 +194,10 @@ LMS7002M::LMS7002M() :
     MemorySectionAddresses[RxGFIR3c][1] = 0x05A7;
     MemorySectionAddresses[RSSI_DC_CALIBRATION][0] = 0x05C0;
     MemorySectionAddresses[RSSI_DC_CALIBRATION][1] = 0x05CC;
+    MemorySectionAddresses[RSSI_PDET_TEMP_CONFIG][0] = 0x0600;
+    MemorySectionAddresses[RSSI_PDET_TEMP_CONFIG][1] = 0x0606;
+    MemorySectionAddresses[RSSI_DC_CONFIG][0] = 0x0640;
+    MemorySectionAddresses[RSSI_DC_CONFIG][1] = 0x0641;
 
     mRegistersMap->InitializeDefaultValues(LMS7parameterList);
     mcuControl = new MCU_BD();
@@ -201,6 +206,8 @@ LMS7002M::LMS7002M() :
 
 LMS7002M::~LMS7002M()
 {
+    if (mValueCache)
+        delete mValueCache;
     delete mcuControl;
     delete mRegistersMap;
 }
@@ -368,7 +375,7 @@ int LMS7002M::ResetChip()
 {
     checkConnection();
 
-    int status = controlPort->DeviceReset();
+    int status = controlPort->DeviceReset(mdevIndex);
     if (status == 0) Modify_SPI_Reg_bits(LMS7param(MIMO_SISO), 0); //enable B channel after reset
     return status;
 }
@@ -392,6 +399,7 @@ int LMS7002M::LoadConfigLegacyFile(const char* filename)
         return ReportError(ENOENT, "LoadConfigLegacyFile(%s) - file not found", filename);
     }
     f.close();
+
     uint16_t addr = 0;
     uint16_t value = 0;
     Channel ch = this->GetActiveChannel(); //remember used channel
@@ -575,6 +583,14 @@ int LMS7002M::LoadConfig(const char* filename)
         return ReportError(ENOENT, "LoadConfig(%s) - file not found", filename);
     }
     f.close();
+
+    if(mcuControl)
+    {
+        mcuControl->RunProcedure(MCU_FUNCTION_GET_PROGRAM_ID);
+        if(mcuControl->WaitForMCU(100) != MCU_ID_CALIBRATIONS_SINGLE_IMAGE)
+            mcuControl->Program_MCU(mcu_program_lms7_dc_iq_calibration_bin, IConnection::MCU_PROG_MODE::SRAM);
+    }
+
     uint16_t addr = 0;
     uint16_t value = 0;
     Channel ch = this->GetActiveChannel(); //remember used channel
@@ -1053,8 +1069,8 @@ float_type LMS7002M::GetFrequencyCGEN()
 float_type LMS7002M::GetReferenceClk_TSP(bool tx)
 {
     float_type cgenFreq = GetFrequencyCGEN();
-	float_type clklfreq = cgenFreq/pow(2.0, Get_SPI_Reg_bits(LMS7param(CLKH_OV_CLKL_CGEN)));
-    if(Get_SPI_Reg_bits(LMS7param(EN_ADCCLKH_CLKGN)) == 0)
+	float_type clklfreq = cgenFreq/pow(2.0, Get_SPI_Reg_bits(LMS7param(CLKH_OV_CLKL_CGEN),true));
+    if(Get_SPI_Reg_bits(LMS7param(EN_ADCCLKH_CLKGN),true) == 0)
         return tx ? clklfreq : cgenFreq/4.0;
     else
         return tx ? cgenFreq : clklfreq/4.0;
@@ -1478,7 +1494,7 @@ int LMS7002M::SetFrequencySX(bool tx, float_type freq_Hz, SX_details* output)
     }
     if(foundInCache)
     {
-        printf("SetFrequency using cache values vco:%i, csw:%i\n", vco_query, csw_query);
+        lime::info("SetFrequency using cache values vco:%i, csw:%i", vco_query, csw_query);
         sel_vco = vco_query;
         csw_value = csw_query;
     }
@@ -2175,7 +2191,7 @@ bool LMS7002M::IsSynced()
         }
         if (dataReceived[i] != regValue)
         {
-            printf("Addr: 0x%04X  gui: 0x%04X  chip: 0x%04X\n", addrToRead[i], regValue, dataReceived[i]);
+            lime::debug("Addr: 0x%04X  gui: 0x%04X  chip: 0x%04X", addrToRead[i], regValue, dataReceived[i]);
             isSynced = false;
             goto isSyncedEnding;
         }
@@ -2214,7 +2230,7 @@ bool LMS7002M::IsSynced()
         }
         if (dataReceived[i] != regValue)
         {
-            printf("Addr: 0x%04X  gui: 0x%04X  chip: 0x%04X\n", addrToRead[i], regValue, dataReceived[i]);
+            lime::debug("Addr: 0x%04X  gui: 0x%04X  chip: 0x%04X", addrToRead[i], regValue, dataReceived[i]);
             isSynced = false;
             goto isSyncedEnding;
         }
@@ -2380,27 +2396,24 @@ int LMS7002M::SetInterfaceFrequency(float_type cgen_freq_Hz, const uint8_t inter
 float_type LMS7002M::GetSampleRate(bool tx, Channel ch)
 {
     float_type interface_Hz;
+    int ratio;
     auto chBck = GetActiveChannel();
     SetActiveChannel(ch);
     //if decimation/interpolation is 0(2^1) or 7(bypass), interface clocks should not be divided
     if (tx)
     {
-        int interpolation = Get_SPI_Reg_bits(LMS7param(HBI_OVR_TXTSP));
-        float_type interfaceTx_Hz = GetReferenceClk_TSP(LMS7002M::Tx);
-        if (interpolation != 7)
-            interfaceTx_Hz /= 2*pow(2.0, interpolation);
-        interface_Hz = interfaceTx_Hz;
+        ratio = Get_SPI_Reg_bits(LMS7param(HBI_OVR_TXTSP),true);
+        interface_Hz = GetReferenceClk_TSP(lime::LMS7002M::Tx);
     }
     else
     {
-        int decimation = Get_SPI_Reg_bits(LMS7param(HBD_OVR_RXTSP));
-        float_type interfaceRx_Hz = GetReferenceClk_TSP(LMS7002M::Rx);
-        if (decimation != 7)
-            interfaceRx_Hz /= 2*pow(2.0, decimation);
-        interface_Hz = interfaceRx_Hz;
+        ratio = Get_SPI_Reg_bits(LMS7param(HBD_OVR_RXTSP),true);
+        interface_Hz = GetReferenceClk_TSP(lime::LMS7002M::Rx);
     }
     SetActiveChannel(chBck);
-    return interface_Hz;
+    if (ratio != 7)
+        interface_Hz /= pow(2.0, ratio);
+    return interface_Hz/2.0;
 }
 
 void LMS7002M::ConfigureLML_RF2BB(
@@ -2538,6 +2551,13 @@ LMS7002M_SelfCalState::~LMS7002M_SelfCalState(void)
 
 void LMS7002M::EnableValuesCache(bool enabled)
 {
+    if (mValueCache && (!enabled))
+    {
+        delete mValueCache;
+        mValueCache = nullptr;
+    }
+    else if (mValueCache == nullptr && enabled)
+        mValueCache = new CalibrationCache();
     useCache = enabled;
 }
 
@@ -2558,9 +2578,9 @@ void LMS7002M::EnableCalibrationByMCU(bool enabled)
 
 float_type LMS7002M::GetTemperature()
 {
+    CalibrateInternalADC(32);
     Modify_SPI_Reg_bits(LMS7_RSSI_PD, 0);
     Modify_SPI_Reg_bits(LMS7_RSSI_RSSIMODE, 0);
-    Modify_SPI_Reg_bits(LMS7_DAC_CLKDIV, 32);
     uint16_t biasMux = Get_SPI_Reg_bits(LMS7_MUX_BIAS_OUT);
     Modify_SPI_Reg_bits(LMS7_MUX_BIAS_OUT, 2);
 
@@ -2574,7 +2594,7 @@ float_type LMS7002M::GetTemperature()
     Vdiff /= 3.9;
     float temperature = 40.5+Vdiff;
     Modify_SPI_Reg_bits(LMS7_MUX_BIAS_OUT, biasMux);
-    printf("Vtemp 0x%04X, Vptat 0x%04X, Vdiff = %.2f, temp= %.3f\n", (reg606 >> 8) & 0xFF, reg606 & 0xFF, Vdiff, temperature);
+    lime::debug("Vtemp 0x%04X, Vptat 0x%04X, Vdiff = %.2f, temp= %.3f", (reg606 >> 8) & 0xFF, reg606 & 0xFF, Vdiff, temperature);
     return temperature;
 }
 
@@ -2609,9 +2629,18 @@ int LMS7002M::CopyChannelRegisters(const Channel src, const Channel dest, const
 
 int LMS7002M::CalibrateAnalogRSSI_DC_Offset()
 {
+    Modify_SPI_Reg_bits(LMS7_EN_INSHSW_W_RFE, 1);
+    CalibrateInternalADC(0);
     Modify_SPI_Reg_bits(LMS7param(PD_RSSI_RFE), 0);
     Modify_SPI_Reg_bits(LMS7param(PD_TIA_RFE), 0);
-    Modify_SPI_Reg_bits(LMS7param(RSSIDC_RSEL), 26);
+
+    /*Modify_SPI_Reg_bits(LMS7param(RSSIDC_RSEL), 22);
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_HYSCMP), 0);
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_PD), 0);*/
+    SPI_write(0x0640, 22 << 4);
+
+    Modify_SPI_Reg_bits(LMS7param(RSSIDC_DCO2), 0);
+
     int value = -63;
     uint8_t wrValue = abs(value);
     if(value < 0)
@@ -2638,7 +2667,7 @@ int LMS7002M::CalibrateAnalogRSSI_DC_Offset()
     }
     if(edges.size() != 2)
     {
-        printf("Not found\n");
+        lime::debug("Not found");
         return ReportError(EINVAL, "Failed to find value");
     }
     int8_t found = (edges[0]+edges[1])/2;
@@ -2646,6 +2675,7 @@ int LMS7002M::CalibrateAnalogRSSI_DC_Offset()
     if(found < 0)
         wrValue |= 0x40;
     Modify_SPI_Reg_bits(LMS7param(RSSIDC_DCO1), wrValue, true);
-    printf("Found %i\n", found);
+    lime::debug("Found %i", found);
+    Modify_SPI_Reg_bits(LMS7_EN_INSHSW_W_RFE, 0);
     return 0;
 }
