@@ -14,6 +14,7 @@
 #include <assert.h>
 #include "MCU_BD.h"
 #include "mcu_programs.h"
+#include "Logger.h"
 static const uint16_t MCU_PARAMETER_ADDRESS = 0x002D; //register used to pass parameter values to MCU
 
 #ifdef _MSC_VER
@@ -122,14 +123,14 @@ int LMS7002M::TuneRxFilter(float_type rx_lpf_freq_RF)
         //set reference clock parameter inside MCU
         long refClk = GetReferenceClk_SX(false);
         mcuControl->SetParameter(MCU_BD::MCU_REF_CLK, refClk);
-        printf("MCU Ref. clock: %g MHz\n", refClk / 1e6);
+        lime::debug("MCU Ref. clock: %g MHz", refClk / 1e6);
         //set bandwidth for MCU to read from register, value is integer stored in MHz
         mcuControl->SetParameter(MCU_BD::MCU_BW, rx_lpf_freq_RF);
         mcuControl->RunProcedure(5);
         status = mcuControl->WaitForMCU(1000);
         if(status != 0)
         {
-            printf("MCU working too long %i\n", status);
+            lime::error("MCU working too long %i", status);
         }
         //sync registers to cache
         for (int a = 0x010c; a <= 0x0114; a++) this->SPI_read(a, true);
@@ -898,14 +899,14 @@ int LMS7002M::TuneTxFilter(const float_type tx_lpf_freq_RF)
         //set reference clock parameter inside MCU
         long refClk = GetReferenceClk_SX(false);
         mcuControl->SetParameter(MCU_BD::MCU_REF_CLK, refClk);
-        printf("MCU Ref. clock: %g MHz\n", refClk / 1e6);
+        lime::debug("MCU Ref. clock: %g MHz", refClk / 1e6);
         //set bandwidth for MCU to read from register, value is integer stored in MHz
         mcuControl->SetParameter(MCU_BD::MCU_BW, tx_lpf_freq_RF);
         mcuControl->RunProcedure(6);
         status = mcuControl->WaitForMCU(1000);
         if(status != 0)
         {
-            printf("MCU working too long %i\n", status);
+            lime::error("MCU working too long %i", status);
         }
         //sync registers to cache
         for (int a = 0x0105; a <= 0x010b; a++) this->SPI_read(a, true);
@@ -1100,6 +1101,9 @@ static int TBB_LPFH_FILTNUM = 600;
 
 int LMS7002M::TuneTxFilterWithCaching(const float_type bandwidth)
 {
+    if (!useCache)
+        return TuneTxFilter(bandwidth);
+    
     int ret = 0;
     bool found = true;
     const int idx = this->GetActiveChannelIndex();
@@ -1112,7 +1116,6 @@ int LMS7002M::TuneTxFilterWithCaching(const float_type bandwidth)
     const bool lpfladFound = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Tx, TBB_LPFLAD_FILTNUM, &rcal_lpflad_tbb, &ccal_lpflad_tbb) == 0);
     const bool lpfs5Found = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Tx, TBB_LPFS5_FILTNUM, &rcal_lpfs5_tbb, &ccal_lpflad_tbb) == 0);
     const bool lpfhFound = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Tx, TBB_LPFH_FILTNUM, &rcal_lpfh_tbb, &ccal_lpflad_tbb) == 0);
-    if (not useCache) found = false;
 
     //apply the calibration
     SPI_write(0x0106, 0x318C);
@@ -1167,6 +1170,9 @@ int LMS7002M::TuneTxFilterWithCaching(const float_type bandwidth)
 
 int LMS7002M::TuneRxFilterWithCaching(const float_type bandwidth)
 {
+    if(!useCache)
+        return TuneRxFilter(bandwidth);
+
     int ret = 0;
     bool found = true;
     const int idx = this->GetActiveChannelIndex();
@@ -1179,7 +1185,6 @@ int LMS7002M::TuneRxFilterWithCaching(const float_type bandwidth)
     const bool tiaFound = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Rx, RFE_TIA_FILTNUM, &rcomp_tia_rfe, &ccomp_tia_rfe, &cfb_tia_rfe) == 0);
     const bool lphlFound = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Rx, RBB_LPFL_FILTNUM, &rcc_ctl_lpfl_rbb, &c_ctl_lpfl_rbb) == 0);
     const bool lphhFound = (mValueCache->GetFilter_RC(boardId, bandwidth, idx, Rx, RBB_LPFH_FILTNUM, &rcc_ctl_lpfh_rbb, &c_ctl_lpfh_rbb) == 0);
-    if (not useCache) found = false;
 
     //apply the calibration
     if (not tiaFound) found = false;
