@@ -742,7 +742,7 @@ static const struct LMS7Parameter LMS7_RSSI_PDET2_VAL = { 0x0604, 15, 8, 0, "RSS
 static const struct LMS7Parameter LMS7_RSSI_PDET1_VAL = { 0x0604, 7, 0, 0, "RSSI_PDET1_VAL", "Stores the value of Power Detector 1"};
 static const struct LMS7Parameter LMS7_RSSI_RSSI2_VAL = { 0x0605, 15, 8, 0, "RSSI_RSSI2_VAL", "Stores the value of RSSI 2"};
 static const struct LMS7Parameter LMS7_RSSI_RSSI1_VAL = { 0x0605, 7, 0, 0, "RSSI_RSSI1_VAL", "Stores the value of RSSI 1"};
-static const struct LMS7Parameter LMS7_RSSI_TREF_VAL = { 0x0606, 15, 8, 0, "RSSI_TREF_CAL", "Stores the temperature reference value"};
+static const struct LMS7Parameter LMS7_RSSI_TREF_VAL = { 0x0606, 15, 8, 0, "RSSI_TREF_VAL", "Stores the temperature reference value"};
 static const struct LMS7Parameter LMS7_RSSI_TVPTAT_VAL = { 0x0606, 7, 0, 0, "RSSI_TVPTAT_VAL", "Stores the voltage proportional to absolute temperature value"};
 static const struct LMS7Parameter LMS7_RSSIDC_CMPSTATUS = { 0x0640, 15, 15, 0, "RSSIDC_CMPSTATUS", ""};
 static const struct LMS7Parameter LMS7_RSSIDC_RSEL = { 0x0640, 8, 4, 10, "RSSIDC_RSEL", "Reference voltage for the RSSI output comparator"};
