@@ -8,7 +8,7 @@
 
 using namespace lime;
 
-static const int MAX_CHANNEL_COUNT = 4;
+static const int MAX_CHANNEL_COUNT = 6;
 
 ILimeSDRStreaming::ILimeSDRStreaming()
 {
@@ -73,7 +73,8 @@ int ILimeSDRStreaming::WriteStream(const size_t streamID, const void* buffs, con
     lime::IStreamChannel* channel = (lime::IStreamChannel*)streamID;
     lime::IStreamChannel::Metadata meta;
     meta.flags = 0;
-    meta.flags |= metadata.hasTimestamp ? lime::IStreamChannel::Metadata::SYNC_TIMESTAMP : 0;
+    meta.flags = metadata.hasTimestamp ? lime::IStreamChannel::Metadata::SYNC_TIMESTAMP : 0;
+    meta.flags |= metadata.endOfBurst ? lime::IStreamChannel::Metadata::END_BURST : 0;
     meta.timestamp = metadata.timestamp;
     int status = channel->Write(buffs, length, &meta, timeout_ms);
     return status;
@@ -142,26 +143,58 @@ int ILimeSDRStreaming::SendData(const char* buffer, int length, int epIndex, int
 
 int ILimeSDRStreaming::UploadWFM(const void* const* samples, uint8_t chCount, size_t sample_count, StreamConfig::StreamDataFormat format, int epIndex)
 {
+    bool comp = true;
+
+    switch (GetInfo().device)
+    {
+        case LMS_DEV_STREAM:
+        case LMS_DEV_LIMESDR:
+        case LMS_DEV_LIMESDR_PCIE:
+        case LMS_DEV_LMS7002M_ULTIMATE_EVB:
+            break;
+        case LMS_DEV_LIMESDR_QPCIE:
+            if (epIndex == 2)
+                comp = (format == StreamConfig::STREAM_12_BIT_COMPRESSED);
+            break;
+        default: return ReportError("UploadWFM not supported");
+    }
+
+    const int samplesInPkt = comp ? 1360 : 1020;
+    WriteRegister(0xFFFF, 1 << epIndex);
     WriteRegister(0x000C, chCount == 2 ? 0x3 : 0x1); //channels 0,1
-    WriteRegister(0x000E, 0x2); //12bit samples
+    WriteRegister(0x000E, comp ? 0x2 : 0x0); //16bit samples
 
     uint16_t regValue = 0;
     ReadRegister(0x000D,regValue);
-    regValue |= 0x4 << (epIndex*2);
+    regValue |= 0x4;
     WriteRegister(0x000D, regValue);
 
     lime::FPGA_DataPacket pkt;
     size_t samplesUsed = 0;
+    int cnt = sample_count;
+
+    const complex16_t* const* src = (const complex16_t* const*)samples;
+    const lime::complex16_t** batch = new const lime::complex16_t*[chCount];
     lime::complex16_t** samplesShort = new lime::complex16_t*[chCount];
     for(unsigned i=0; i<chCount; ++i)
         samplesShort[i] = nullptr;
 
-    const complex16_t* const* src = format == StreamConfig::STREAM_COMPLEX_FLOAT32 ? samplesShort :(const complex16_t* const*)samples;
-    int cnt = sample_count;
-    const lime::complex16_t** batch = new const lime::complex16_t*[chCount];
 
-    if(format == StreamConfig::STREAM_COMPLEX_FLOAT32)
+    if (format == StreamConfig::STREAM_12_BIT_IN_16 && comp == true)
     {
+        for(unsigned i=0; i<chCount; ++i)
+            samplesShort[i] = new lime::complex16_t[sample_count];
+        for (int ch = 0; ch < chCount; ch++)
+            for(size_t i=0; i < sample_count; ++i)
+            {
+                samplesShort[ch][i].i = src[ch][i].i >> 4;
+                samplesShort[ch][i].q = src[ch][i].q >> 4;
+            }
+        src = samplesShort;
+    }
+    else if(format == StreamConfig::STREAM_COMPLEX_FLOAT32)
+    {
+        const float mult = comp ? 2047.0f : 32767.0f;
         for(unsigned i=0; i<chCount; ++i)
             samplesShort[i] = new lime::complex16_t[sample_count];
 
@@ -169,24 +202,23 @@ int ILimeSDRStreaming::UploadWFM(const void* const* samples, uint8_t chCount, si
         for (int ch = 0; ch < chCount; ch++)
             for(size_t i=0; i < sample_count; ++i)
             {
-                samplesShort[ch][i].i = samplesFloat[ch][2*i]*2047.0f;
-                samplesShort[ch][i].q = samplesFloat[ch][2*i+1]*2047.0f;
+                samplesShort[ch][i].i = samplesFloat[ch][2*i]*mult;
+                samplesShort[ch][i].q = samplesFloat[ch][2*i+1]*mult;
             }
+        src = samplesShort;
     }
 
     while(cnt > 0)
     {
         pkt.counter = 0;
         pkt.reserved[0] = 0;
-        int samplesToSend = cnt > 1360/chCount ? 1360/chCount : cnt;
-        cnt -= samplesToSend;
+        int samplesToSend = cnt > samplesInPkt/chCount ? samplesInPkt/chCount : cnt;
 
         for(unsigned i=0; i<chCount; ++i)
             batch[i] = &src[i][samplesUsed];
         samplesUsed += samplesToSend;
 
-        size_t bufPos = 0;
-        lime::fpga::Samples2FPGAPacketPayload(batch, samplesToSend, chCount, StreamConfig::STREAM_12_BIT_COMPRESSED, pkt.data, &bufPos);
+        int bufPos = lime::fpga::Samples2FPGAPacketPayload(batch, samplesToSend, chCount==2, comp, pkt.data);
         int payloadSize = (bufPos / 4) * 4;
         if(bufPos % 4 != 0)
             lime::error("Packet samples count not multiple of 4");
@@ -197,6 +229,7 @@ int ILimeSDRStreaming::UploadWFM(const void* const* samples, uint8_t chCount, si
         long bToSend = 16+payloadSize;
         if (SendData((const char*)&pkt,bToSend,epIndex,500)!=bToSend)
             break;
+        cnt -= samplesToSend;
     }
     delete[] batch;
     for(unsigned i=0; i<chCount; ++i)
@@ -205,7 +238,7 @@ int ILimeSDRStreaming::UploadWFM(const void* const* samples, uint8_t chCount, si
     delete[] samplesShort;
 
     /*Give FX3 some time to load samples to FPGA*/
-    std::this_thread::sleep_for(std::chrono::milliseconds(300));
+    std::this_thread::sleep_for(std::chrono::milliseconds(500));
     if(cnt == 0)
         return 0;
     else
@@ -222,6 +255,8 @@ ILimeSDRStreaming::StreamChannel::StreamChannel(Streamer* streamer, StreamConfig
     overflow = 0;
     underflow = 0;
     pktLost = 0;
+    sampleCnt = 0;
+    startTime = std::chrono::high_resolution_clock::now();
 
     if (this->config.bufferLength == 0) //default size
         this->config.bufferLength = 1024*8*SamplesPacket::maxSamplesInPacket;
@@ -251,7 +286,7 @@ int ILimeSDRStreaming::StreamChannel::Read(void* samples, const uint32_t count,
         float* samplesFloat = (float*)samples;
         popped = fifo->pop_samples(ptr, count, 1, &meta->timestamp, timeout_ms, &meta->flags);
         for(int i=2*popped-1; i>=0; --i)
-            samplesFloat[i] = (float)samplesShort[i]/2048.0;
+            samplesFloat[i] = (float)samplesShort[i]/32767.0f;
     }
     else
     {
@@ -264,22 +299,24 @@ int ILimeSDRStreaming::StreamChannel::Read(void* samples, const uint32_t count,
 int ILimeSDRStreaming::StreamChannel::Write(const void* samples, const uint32_t count, const Metadata *meta, const int32_t timeout_ms)
 {
     int pushed = 0;
+    if (config.isTx && mActive && mStreamer->txRunning.load() == false)
+        mStreamer->UpdateThreads();
     if(config.format == StreamConfig::STREAM_COMPLEX_FLOAT32 && config.isTx)
     {
         const float* samplesFloat = (const float*)samples;
         int16_t* samplesShort = new int16_t[2*count];
         for(size_t i=0; i<2*count; ++i)
-            samplesShort[i] = samplesFloat[i]*2047;
+            samplesShort[i] = samplesFloat[i]*32767.0f;
         const complex16_t* ptr = (const complex16_t*)samplesShort ;
         pushed = fifo->push_samples(ptr, count, 1, meta->timestamp, timeout_ms, meta->flags);
         delete[] samplesShort;
     }
-    //else if(config.format == StreamConfig::STREAM_12_BIT_IN_16)
     else
     {
         const complex16_t* ptr = (const complex16_t*)samples;
         pushed = fifo->push_samples(ptr, count, 1, meta->timestamp, timeout_ms, meta->flags);
     }
+    sampleCnt += pushed;
     return pushed;
 }
 
@@ -294,6 +331,11 @@ IStreamChannel::Info ILimeSDRStreaming::StreamChannel::GetInfo()
     stats.droppedPackets = pktLost;
     stats.overrun = overflow;
     stats.overrun = underflow;
+    auto endTime = std::chrono::high_resolution_clock::now();
+    std::chrono::duration<double> timePeriod = endTime-startTime;
+    stats.sampleRate = sampleCnt/timePeriod.count();
+    sampleCnt.store(0);
+    startTime = endTime; 
     pktLost = 0;
     overflow = 0;
     underflow = 0;
@@ -316,6 +358,8 @@ int ILimeSDRStreaming::StreamChannel::Start()
     overflow = 0;
     underflow = 0;
     pktLost = 0;
+    sampleCnt.store(0);
+    startTime = std::chrono::high_resolution_clock::now();
     return mStreamer->UpdateThreads();
 }
 
@@ -336,9 +380,10 @@ ILimeSDRStreaming::Streamer::Streamer(ILimeSDRStreaming* port)
     terminateTx = false;
     rxRunning = false;
     txRunning = false;
-    generateData = false;
     rxDataRate_Bps = 0;
     txDataRate_Bps = 0;
+    txBatchSize = 1;
+    rxBatchSize = 1;
     mChipID = dataPort->mStreamers.size();
 }
 
@@ -348,12 +393,18 @@ ILimeSDRStreaming::Streamer::~Streamer()
         CloseStream((size_t)i);
     for(auto i : mRxStreams)
         CloseStream((size_t)i);
+    terminateTx.store(true);
+    if (txThread.joinable())
+        txThread.join();
+    terminateRx.store(true);
+    if (rxThread.joinable())
+        rxThread.join();
 }
 
 int ILimeSDRStreaming::Streamer::SetupStream(size_t& streamID, const StreamConfig& config)
 {
-    if(rxRunning.load() == true || txRunning.load() == true)
-        return ReportError(EPERM, "All streams must be stopped before doing setups");
+    /*if(rxRunning.load() == true || txRunning.load() == true)
+        return ReportError(EPERM, "All streams must be stopped before doing setups");*/
     streamID = ~0;
     StreamChannel* stream = new StreamChannel(this,config);
     //TODO check for duplicate streams
@@ -362,6 +413,22 @@ int ILimeSDRStreaming::Streamer::SetupStream(size_t& streamID, const StreamConfi
     else
         mRxStreams.push_back(stream);
     streamID = size_t(stream);
+    LMS7002M lms;
+    lms.SetConnection(dataPort, mChipID);
+    double rate = lms.GetSampleRate(config.isTx,LMS7002M::ChA)/1e6;
+    int size = (config.isTx) ?  mTxStreams.size(): mRxStreams.size();
+
+    if (config.performanceLatency < 0.5)
+        rate = 0.5 + rate * config.performanceLatency * 2.0 * size;
+    else
+        rate += (config.performanceLatency - 0.5) * 40.0 * size;
+
+    for (int batch = 1; batch < rate; batch <<= 1)
+        if (config.isTx)
+            txBatchSize = batch;
+        else
+            rxBatchSize = batch;
+
     return 0; //success
 }
 
@@ -415,15 +482,12 @@ void ILimeSDRStreaming::Streamer::EnterSelfCalibration()
 {
     if(not rxRunning)
         return;
-    generateData.store(true);
-    std::unique_lock<std::mutex> lck(streamStateLock);
-    //wait untill all existing USB transfers complete
-    safeToConfigInterface.wait_for(lck, std::chrono::milliseconds(250));
+    UpdateThreads(true);
 }
 
 void ILimeSDRStreaming::Streamer::ExitSelfCalibration()
 {
-    generateData.store(false);
+    UpdateThreads();
 }
 
 uint64_t ILimeSDRStreaming::Streamer::GetHardwareTimestamp(void)
@@ -431,8 +495,9 @@ uint64_t ILimeSDRStreaming::Streamer::GetHardwareTimestamp(void)
     if(not rxRunning.load() and not txRunning.load())
     {
         //stop streaming just in case the board has not been configured
-        fpga::StopStreaming(dataPort, mChipID);
-        fpga::ResetTimestamp(dataPort, mChipID);
+        dataPort->WriteRegister(0xFFFF, 1 << mChipID);
+        fpga::StopStreaming(dataPort);
+        fpga::ResetTimestamp(dataPort);
         mTimestampOffset = 0;
         return 0;
     }
@@ -482,15 +547,15 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
         rxThread.join();
         rxRunning.store(false);
     }
-
+    dataPort->WriteRegister(0xFFFF, 1 << mChipID);
     //configure FPGA on first start, or disable FPGA when not streaming
     if((needTx or needRx) && (not rxRunning.load() and not txRunning.load()))
     {
         LMS7002M lmsControl;
         lmsControl.SetConnection(dataPort, mChipID);
         //enable FPGA streaming
-        fpga::StopStreaming(dataPort, mChipID);
-        fpga::ResetTimestamp(dataPort, mChipID);
+        fpga::StopStreaming(dataPort);
+        fpga::ResetTimestamp(dataPort);
         rxLastTimestamp.store(0);
         //Clear device stream buffers
         dataPort->ResetStreamBuffers();
@@ -502,7 +567,7 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
 
         for(auto i : mRxStreams)
         {
-            if(i->config.format == StreamConfig::STREAM_12_BIT_IN_16)
+            if(i->config.format != StreamConfig::STREAM_12_BIT_COMPRESSED)
             {
                 config.linkFormat = StreamConfig::STREAM_12_BIT_IN_16;
                 break;
@@ -510,7 +575,7 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
         }
         for(auto i : mTxStreams)
         {
-            if(i->config.format == StreamConfig::STREAM_12_BIT_IN_16)
+            if(i->config.format != StreamConfig::STREAM_12_BIT_COMPRESSED)
             {
                 config.linkFormat = StreamConfig::STREAM_12_BIT_IN_16;
                 break;
@@ -528,7 +593,7 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
         else if(config.linkFormat == StreamConfig::STREAM_12_BIT_COMPRESSED)
             smpl_width = 0x2;
         else
-            smpl_width = 0x2;
+            smpl_width = 0x0;
 
         if (lmsControl.Get_SPI_Reg_bits(LMS7param(LML1_SISODDR),true))
             mode = 0x0040;
@@ -580,12 +645,12 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
             lmsControl.Modify_SPI_Reg_bits(LMS7param(MAC), macBck, fromChip);
         }
 
-        fpga::StartStreaming(dataPort, mChipID);
+        fpga::StartStreaming(dataPort);
     }
     else if(not needTx and not needRx)
     {
         //disable FPGA streaming
-        fpga::StopStreaming(dataPort, mChipID);
+        fpga::StopStreaming(dataPort);
     }
 
     //FPGA should be configured and activated, start needed threads
@@ -597,6 +662,8 @@ int ILimeSDRStreaming::Streamer::UpdateThreads(bool stopAll)
     }
     if(needTx and not txRunning.load())
     {
+        if (txThread.joinable())
+            txThread.join();
         txRunning.store(true);
         terminateTx.store(false);
         txThread = std::thread(dataPort->TxLoopFunction, this);
