@@ -6,6 +6,7 @@
 #include <mutex>
 #include <condition_variable>
 #include <vector>
+#include <chrono>
 
 #include "dataTypes.h"
 #include "fifo.h"
@@ -47,6 +48,8 @@ public:
     protected:
         RingFIFO* fifo;
         bool mActive;
+        std::atomic<uint64_t> sampleCnt;
+        std::chrono::time_point<std::chrono::high_resolution_clock> startTime;
     private:
         StreamChannel() = default;
     };
@@ -76,9 +79,6 @@ public:
         std::atomic<bool> txRunning;
         std::atomic<bool> terminateRx;
         std::atomic<bool> terminateTx;
-        std::mutex streamStateLock;
-        std::condition_variable safeToConfigInterface;
-        std::atomic<bool> generateData;
 
         std::vector<StreamChannel*> mRxStreams;
         std::vector<StreamChannel*> mTxStreams;
@@ -86,6 +86,8 @@ public:
         std::atomic<uint64_t> txLastLateTime;
         uint64_t mTimestampOffset;
         int mChipID;
+        unsigned txBatchSize;
+        unsigned rxBatchSize;
     };
 
     ILimeSDRStreaming();
