@@ -51,7 +51,7 @@ LMS64CProtocol::~LMS64CProtocol(void)
     return;
 }
 
-int LMS64CProtocol::DeviceReset(void)
+int LMS64CProtocol::DeviceReset(int ind)
 {
     if (not this->IsOpen())
     {
@@ -60,6 +60,7 @@ int LMS64CProtocol::DeviceReset(void)
 
     GenericPacket pkt;
     pkt.cmd = CMD_LMS7002_RST;
+    pkt.periphID = ind;
     pkt.outBuffer.push_back (LMS_RST_PULSE);
     int status = this->TransferPacket(pkt);
 
@@ -418,10 +419,7 @@ int LMS64CProtocol::TransferPacket(GenericPacket& pkt)
     int outBufPos = 0;
     int inDataPos = 0;
     if(outLen == 0)
-    {
-        //printf("packet outlen = 0\n");
         outLen = 1;
-    }
 
     if(protocol == LMS_PROTOCOL_NOVENA)
     {
